FILESEXTRAPATHS_prepend := "${THISDIR}/${PN}:${BSPDIR}/sources/meta-browser/recipes-browser/chromium/chromium:"

PACKAGECONFIG = "use-egl disable-api-keys-info-bar"

include ${BSPDIR}/sources/meta-browser/recipes-browser/chromium/chromium.inc

SRC_URI += "\
    file://Chromium \
    file://init.d/chromium \
    file://chromium.conf \
    file://virtualkeyboard.tar;unpack=false \
"
inherit update-rc.d

LIC_FILES_CHKSUM = "file://LICENSE;md5=537e0b52077bf0a616d0a0c8a79bc9d5"

OZONE_WAYLAND_GIT_BRANCH = "Milestone-ThanksGiving"
OZONE_WAYLAND_GIT_SRCREV = "5d7baa9bc3b8c88e9b7e476e3d6bc8cd44a887fe"
# using 00*.patch to skip the WebRTC patches in ozone-wayland
# the WebRTC patches remove X11 libraries from the linker flags, which is
# already done by another patch (see above). Furthermore, to be able to use
# these patches, it is necessary to update the git repository in third_party/webrtc,
# which would further complicate this recipe.
OZONE_WAYLAND_PATCH_FILE_GLOB = "00*.patch"

python() {
	# Component build is broken in ozone-wayland for Chromium
    if (d.getVar('ENABLE_X11', True) != '1') and (d.getVar('ENABLE_WAYLAND', True) == '1'):
        if bb.utils.contains('PACKAGECONFIG', 'component-build', True, False, d):
            bb.fatal("Chromium ${PV} Wayland version cannot be built in component-mode")
}

do_fetch_prepend() {
    if (d.getVar('DEFAULT_PREFERENCE', True) == '-1') :
        bb.warn("${P} is currently not recommended for release (probably beta or development version)")
}

do_install_append() {
	install -d ${D}${bindir}
	install -m 0755 ${WORKDIR}/Chromium ${D}${bindir}/

	install -m 0644 ${S}/out/Release/locales/en-GB.pak ${D}${bindir}/chrome/locales
	install -m 0644 ${S}/out/Release/locales/de.pak ${D}${bindir}/chrome/locales
	install -m 0644 ${S}/out/Release/locales/es.pak ${D}${bindir}/chrome/locales
	install -m 0644 ${S}/out/Release/locales/es-419.pak ${D}${bindir}/chrome/locales
	install -m 0644 ${S}/out/Release/locales/fr.pak ${D}${bindir}/chrome/locales
	install -m 0644 ${S}/out/Release/locales/ru.pak ${D}${bindir}/chrome/locales
	install -m 0644 ${S}/out/Release/locales/ja.pak ${D}${bindir}/chrome/locales
	install -m 0644 ${S}/out/Release/locales/zh-CN.pak ${D}${bindir}/chrome/locales
	install -m 0644 ${S}/out/Release/locales/zh-TW.pak ${D}${bindir}/chrome/locales

	# Init script
	install -d ${D}${sysconfdir}/init.d
	install -m 0755 ${WORKDIR}/init.d/chromium ${D}${sysconfdir}/init.d/chromium
	install -m 0644 ${WORKDIR}/chromium.conf ${D}${sysconfdir}/

	install -d ${D}${ROOT_HOME}/.chromium
	install -m 0644 ${WORKDIR}/virtualkeyboard.tar ${D}${ROOT_HOME}/.chromium/virtualkeyboard.tar
}

FILES_${PN} += "${bindir}/Chromium ${sysconfdir}/chromium.conf ${sysconfdir}/init.d/chromium ${ROOT_HOME}/.chromium/virtualkeyboard.tar"

do_release() {
	_PACKAGE_ARCH=`echo ${PACKAGE_ARCH}|tr '-' '_'`
    cp ${PKGWRITEDIRRPM}/${_PACKAGE_ARCH}/${PF}*${_PACKAGE_ARCH}.rpm ${DEPLOY_DIR_RELEASE_IMAGES}/${RELEASE_PREFIX}${PF}.rpm
}
addtask release after do_package_write_rpm 
do_release[dirs] = "${DEPLOY_DIR_RELEASE_IMAGES}"
do_release[recrdeptask] = "do_release do_build"

# Turn off the qt4-guf-demo

pkg_postinst_${PN} () {
#!/bin/sh -e
if [ -n "$D" ]; then
    OPT="-r $D -f"
else
    OPT="-s -f"
fi
update-rc.d $OPT qt4-guf-demo remove
}

pkg_postrm_${PN} () {
#!/bin/sh -e
if [ -n "$D" ]; then
    OPT="-r $D"
else
    OPT="-s"
fi
update-rc.d $OPT qt4-guf-demo defaults 95 5
}


INITSCRIPT_NAME = "chromium"
INITSCRIPT_PARAMS = "defaults 95 5"

