
MODULES_FSTYPE = "ext4"
MODULES_IMAGE_LINK = "${DEPLOY_DIR_IMAGE}/modules-${MACHINE}.${MODULES_FSTYPE}"
MODULES_IMAGE = "${DEPLOY_DIR_IMAGE}/modules-${MACHINE}-${DATETIME}.${MODULES_FSTYPE}"
MODULES_FOLDER = "${IMAGE_ROOTFS}/lib/modules"
MODULES_CONF_FOLDER = "${IMAGE_ROOTFS}/etc/modules-load.d"

do_deploy_kernelmodules() {
	# Create modules.conf (used as /etc/modules in Flash-N-Go System)
	MODULES_KERNEL_FOLDER=$(find ${MODULES_FOLDER} -maxdepth 1 -mindepth 1)
    if [ -d ${MODULES_CONF_FOLDER}/ ]
    then
    	cat ${MODULES_CONF_FOLDER}/* > ${MODULES_KERNEL_FOLDER}/modules.conf
    else
        touch ${MODULES_KERNEL_FOLDER}/modules.conf
    fi

	# Create filesystem
	MODULES_SIZE=$(du -s ${MODULES_FOLDER} | cut -f1)
	# Add 1MB for journal and 10% overhead for filesystem
	MODULES_FS_SIZE=$(echo "scale=0; 1024+${MODULES_SIZE}*11/10+1" | bc)
	dd if=/dev/zero of=${MODULES_IMAGE} seek=${MODULES_FS_SIZE} count=0 bs=1k
	mkfs.${MODULES_FSTYPE} -F ${MODULES_IMAGE} -d ${MODULES_FOLDER} -m 0
	ln -sf ${MODULES_IMAGE} ${MODULES_IMAGE_LINK}
	rm ${MODULES_KERNEL_FOLDER}/modules.conf
}
addtask deploy_kernelmodules after do_rootfs before do_release
do_deploy_kernelmodules[depends] = "e2fsprogs-native:do_populate_sysroot"

do_release() {
	for fstype in ${IMAGE_FSTYPES}; do
		ln -Lf ${DEPLOY_DIR_IMAGE}/${IMAGE_BASENAME}-${MACHINE}.${fstype} ${DEPLOY_DIR_RELEASE_IMAGES}/${RELEASE_PREFIX}root.${fstype}
	done
	cp ${yocto_packets} ${DEPLOY_DIR_RELEASE_IMAGES}/${RELEASE_PREFIX}standard-packets.html
	cp ${guf_packets} ${DEPLOY_DIR_RELEASE_IMAGES}/${RELEASE_PREFIX}guf-packets.html
	cp ${MODULES_IMAGE_LINK} ${DEPLOY_DIR_RELEASE_IMAGES}/${RELEASE_PREFIX}modules.${MODULES_FSTYPE}
}
addtask release after do_rootfs 
do_release[dirs] = "${DEPLOY_DIR_RELEASE_IMAGES}"
do_release[recrdeptask] = "do_release do_build"

do_release_sdk() {
	ln -Lf ${SDK_DEPLOY}/${TOOLCHAIN_OUTPUTNAME}.sh ${DEPLOY_DIR_RELEASE_SDK}/${RELEASE_PREFIX}sdk.sh
}
addtask release_sdk after do_populate_sdk
do_release_sdk[dirs] = "${DEPLOY_DIR_RELEASE_SDK}"
