#!/bin/sh
#
# Called from udev
#
# Attempt to mount any added block devices and umount any removed devices


MOUNT="/bin/mount"
PMOUNT="/usr/bin/pmount"
UMOUNT="/bin/umount"
for line in `grep -v ^# /etc/udev/mount.blacklist`
do
	if [ ` expr match "$DEVNAME" "$line" ` -gt 0 ];
	then
		logger "udev/mount.sh" "[$DEVNAME] is blacklisted, ignoring"
		exit 0
	fi
done

automount() {	
	name="`basename "$DEVNAME"`"

	! test -d "/media/$name" && mkdir -p "/media/$name"
	# Silent util-linux's version of mounting auto
	if [ "x`readlink $MOUNT`" = "x/bin/mount.util-linux" ] ;
	then
		MOUNT="$MOUNT -o silent"
	fi
	
	# If filesystem type is vfat, change the ownership group to 'disk', and
	# grant it with  w/r/x permissions.
	case $ID_FS_TYPE in
	vfat|fat)
		MOUNT="$MOUNT -o umask=007,gid=`awk -F':' '/^disk/{print $3}' /etc/group`"
		;;
	# TODO
	*)
		;;
	esac

	if ! $MOUNT -t auto -o sync $DEVNAME "/media/$name"
	then
		#logger "mount.sh/automount" "$MOUNT -t auto $DEVNAME \"/media/$name\" failed!"
		rm_dir "/media/$name"
	else
		echo "mount.sh/automount" "Auto-mount of [/media/$name] successful" > /dev/console
		touch "/tmp/.automount-$name"
	fi
	
	# Check if an autocopy folder exists on the partition
	if [ -d /media/$name/autocopy ]; then
		echo -n "mount.sh/automount" "Found an autocopy folder. Copying files..." > /dev/console
		cp -R /media/$name/autocopy/* /	
		echo "done" > /dev/console
	fi
	
	# Search for autostart files and run them
	if [ -d /media/$name/autostart ]; then
		echo "mount.sh/automount" "Found an autostart folder. Executing files..." > /dev/console
		for file in `find /media/$name/autostart -type f -maxdepth 1 | sort`
		do
			if [ -x $file ]; then
				echo "mount.sh/automount" "Executing $file..." > /dev/console
				$file
			fi
		done
		echo "Finished executing autostart files" > /dev/console
	fi

	# Check if an autoupdate folder exists
	if [ -d /media/$name/autoupdate ]; then
		echo "mount.sh/automount" "Found an update stick. Rebooting into FLASH-N-GO System..." > /dev/console

		# Wait until we reach the final runlevel 
		targetRunlevel=$(cat /etc/inittab | grep initdefault |sed -e 's/id:\([0-5]\):initdefault.*/\1/')
		while ! [[ `/sbin/runlevel | cut -d " " -f 2` == ${targetRunlevel} ]]
		do
			echo "Wait for runlevel ${targetRunlevel}" > /dev/console
			sleep 1
		done

		if [ `bootselect alternative` = "alternative" ]; then
			/sbin/reboot
		else
			echo "mount.sh/automount" "Error: bootmode could not be set" > /dev/console
		fi
	fi
}
	
rm_dir() {
	# We do not want to rm -r populated directories
	if test "`find "$1" | wc -l | tr -d " "`" -lt 2 -a -d "$1"
	then
		! test -z "$1" && rm -r "$1"
	else
		logger "mount.sh/automount" "Not removing non-empty directory [$1]"
	fi
}

# No ID_FS_TYPE for cdrom device, yet it should be mounted
name="`basename "$DEVNAME"`"
[ -e /sys/block/$name/device/media ] && media_type=`cat /sys/block/$name/device/media`

if [ "$ACTION" = "add" ] && [ -n "$DEVNAME" ] && [ -n "$ID_FS_TYPE" -o "$media_type" = "cdrom" ]; then
	if [ -x "$PMOUNT" ]; then
		$PMOUNT $DEVNAME 2> /dev/null
	elif [ -x $MOUNT ]; then
    		$MOUNT $DEVNAME 2> /dev/null
	fi
	
	# If the device isn't mounted at this point, it isn't
	# configured in fstab (note the root filesystem can show up as
	# /dev/root in /proc/mounts, so check the device number too)
	if expr $MAJOR "*" 256 + $MINOR != `stat -c %d /`; then
		grep -q "^$DEVNAME " /proc/mounts || automount
	fi
fi



if [ "$ACTION" = "remove" ] && [ -x "$UMOUNT" ] && [ -n "$DEVNAME" ]; then
	for mnt in `cat /proc/mounts | grep "$DEVNAME" | cut -f 2 -d " " `
	do
		$UMOUNT $mnt
	done
	
	# Remove empty directories from auto-mounter
	name="`basename "$DEVNAME"`"
	test -e "/tmp/.automount-$name" && rm_dir "/media/$name"
fi
