#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>

#include <errno.h>
#include <string.h>

#include <sys/ioctl.h>
#include <linux/types.h>
#include <linux/watchdog.h>

int main(int argc, char *argv[])
{
	int new_interval = 2;
	int interval, bootstatus;
	int count = 0, limit;

	fprintf(stdout, "\nTry to open /dev/watchdog\n" );
	int fd = open("/dev/watchdog", O_WRONLY);
	int ret = 0;
	if (fd == -1) {
		perror("watchdog");
		exit(EXIT_FAILURE);
	}

	fprintf(stdout, "\nWatchdog opened\n" );

	/* Check if last boot is caused by watchdog */
	if (ioctl(fd, WDIOC_GETBOOTSTATUS, &bootstatus) == 0) {
		fprintf(stdout, "Last boot is caused by : %s\n",
				(bootstatus != 0) ? "Watchdog" : "Power-On-Reset");
	} else {
		fprintf(stderr, "Error: Cannot read watchdog status\n");
		exit(EXIT_FAILURE);
	}

	/* Set watchdog interval */
	if (ioctl(fd, WDIOC_SETTIMEOUT, &new_interval) == 0) {
		fprintf(stdout, "Set watchdog interval to %d\n", new_interval);
	} else {
		fprintf(stderr, "Error: Cannot read watchdog interval\n");
		exit(EXIT_FAILURE);
	}

	/* Display current watchdog interval */
	if (ioctl(fd, WDIOC_GETTIMEOUT, &interval) == 0) {
		fprintf(stdout, "Current watchdog interval is %d\n", interval);
	} else {
		fprintf(stderr, "Error: Cannot read watchdog interval\n");
		exit(EXIT_FAILURE);
	}
	limit = interval * 5;
	fprintf(stdout, "Trigger watchdog every second for %d seconds\n", limit );
	while (count < limit) {
		fprintf(stdout, "\r%d   ", count++);fflush(stdout);
		ret = write(fd, "\0", 1);
		if (ret != 1) {
			fprintf(stdout, "Watchdog error on write\n" );
			ret = -1;
			break;
		}
		fsync(fd);
		sleep(1);
	}
	count = 0;
	limit = interval + 10;
	fprintf(stdout, "\nStop triggering watchdog and wait %d seconds for reboot\n", limit);
	while (count < limit) {
		fprintf(stdout, "\r%d    ", count++);fflush(stdout);
		sleep(1);
	}

	fprintf(stdout, "Watchdog didn't triggered\n");
	ret = -1;
	close(fd);
	return ret;
}
