#include "mainview.h"
#include "qgufslider.h"
#include "qdigit.h"
#include "qslideshow.h"
#ifdef BACKLIGHT
#include "qbacklight.h"
#endif
#include <stdio.h>
#include <stdlib.h>
#include <QDesktopWidget>
#include <QMdiSubWindow>
#include <QDir>
#include <QDirIterator>
#include <QTextStream>
#include <QDebug>

#define IMAGES_DIR  "/usr/share/qt4-guf-demo/images/"
#define VIDEO_FILE  "/usr/share/qt4-guf-demo/videos/demovideo"
#define PIC_DELAY   4000    // in ms -> 1000ms = 1s
#define T_BEGINN    10000   // in ms

MainView::MainView(QWidget *parent) : QDialog(parent)
{
	setWindowFlags(windowFlags() | Qt::FramelessWindowHint);

	xResolution =  (unsigned int)QApplication::desktop()->screenGeometry(0).width();
	yResolution =  (unsigned int)QApplication::desktop()->screenGeometry(0).height();
	//if(QApplication::desktop()->numScreens() > 1)
		//isMultiScreen = true;
	//else
		isMultiScreen = false;
		
	setupUi(this);	
}

MainView::~MainView()
{
}

void MainView::restartTL()
{
    QTimer::singleShot(1, timeline, SLOT(stop()));
    QTimer::singleShot(1, timeline, SLOT(start()));
}

void MainView::rebootSystem()
{
    system("reboot");  
}

void MainView::uncheckSlideshowButton()
{
	button_slideshow->setChecked(false);
}
void MainView::checkButton()
{
	QPushButton* button = dynamic_cast<QPushButton*>(sender());
	button->setChecked(true);
}

void MainView::setupUi(QDialog *MainView)
{
    QIcon icon_00;
    QIcon icon_01;
    QIcon icon_02;
    QIcon icon_03;
    QIcon icon_10;
    QIcon icon_11;
    QIcon icon_12;
    QIcon icon_13;
    QIcon icon_20;
    QIcon icon_21;
    QIcon icon_22;
    QIcon icon_23;
    QIcon icon_off;
	QIcon icon_slideshow;
    QIcon icon_video;
    QIcon icon_menu;
    QIcon icon_slider;
    QIcon icon_help;
	QString path;
	QPixmap *buttons_off;
	QPixmap *buttons_on;
	QList<QIcon> icons;
	
	unsigned int header_height = 0;
	unsigned int menu_buttons_x = 0;
	unsigned int menu_buttons_y = 0;
	unsigned int menu_button_width = 0;
	unsigned int menu_button_height = 0;
	unsigned int off_button_width = 0;
	unsigned int off_button_height = 0;
	unsigned int off_button_y_offset = 0;
	unsigned int right_button_width = 0;
	unsigned int right_button_height = 0;
	unsigned int slideshow_button_width = 0;
	unsigned int slideshow_button_height = 0;
	unsigned int slider_x_offset = 0;
	unsigned int slider_y_offset = 0;
	unsigned int slider_width = 0;
	unsigned int slider_height = 0;
	unsigned int slider_x_raw_offset = 0;
	unsigned int slider_y_raw_offset = 0;
	unsigned int digits_x_offset = 0;
	unsigned int digits_y_offset = 0;
	unsigned int digits_width = 0;
	unsigned int digits_height = 0;
	unsigned int digits_1_2_offset = 0;
	unsigned int digits_2_3_offset = 0;
	unsigned int video_x_offset = 0;
	unsigned int video_y_offset = 0;
	unsigned int video_width = 0;
	unsigned int video_height = 0;
	unsigned int background_label_width = 0;
	unsigned int background_label_height = 0;

	unsigned int i;
	unsigned int initial_backlight = 0;
	unsigned int initial_digit1 = 0;
	unsigned int initial_digit2 = 0;
	unsigned int initial_digit3 = 0;

	QDirIterator it(IMAGES_DIR, QDir::Dirs | QDir::NoDotAndDotDot, QDirIterator::NoIteratorFlags);

	/* Get machine information to determine hardware platform */
	QFile cpuInfo(QString("/proc/cpuinfo"));
	QString hardware;
	QString machine;
	if (cpuInfo.open(QIODevice::ReadOnly)) {
		QTextStream textStream(&cpuInfo);
		QString line;
		while (!(line = textStream.readLine()).isNull()) {
			if (line.contains(QString("Hardware")))
				hardware = line.split(QString(":")).at(1).simplified();
		}
	}
	while (it.hasNext()) {
		it.next();
		if (hardware.contains(it.fileName(), Qt::CaseInsensitive) && it.fileName().length() > machine.length())
			machine = it.fileName();
	}
	if (machine.isNull())
	{
		qCritical() << "ERROR: Could not find a subfolder of" << IMAGES_DIR << "matching the machine name";
		exit(1);
	}

	QString folder(QString(IMAGES_DIR).append(machine));

	/* The images for the platform (CUPID, ...) are automatically copied
	   to the image folder during the build. */
	if(xResolution >= 1920 && yResolution >= 1080 &&
		QDir(folder + "/1920x1080/").exists())
	{
		xResolution = 1920;
		yResolution = 1080;
		path = folder + "/1920x1080/";
		header_height = 236;
		menu_buttons_x = 4;
		menu_buttons_y = 2;
		menu_button_width = 384;
		menu_button_height = 301;
		off_button_width = 79;
		off_button_height = 238;
		slideshow_button_width = 304;
		slideshow_button_height = 240;
		right_button_width = 384;
		right_button_height = 150;
		slider_x_offset = 328;
		slider_y_offset = 603;
		slider_width = 873;
		slider_height = 202;
		slider_x_raw_offset = 52;
		slider_y_raw_offset = 22;
		background_label_width = 1536;
		background_label_height = 1080;
		digits_x_offset = 470;
		digits_y_offset = 292;
		digits_width = 110;
		digits_height = 162;
		digits_1_2_offset = 7;
		digits_2_3_offset = 60;
		video_x_offset = 0;
		video_y_offset = 242;
		video_width = 1536;
		video_height = 768;
	} else if(xResolution >= 1280 && yResolution >= 800 &&
		QDir(folder + "/1280x800/").exists())
	{
		xResolution = 1280;
		yResolution = 800;
		path = folder + "/1280x800/";
		header_height = 114;
		menu_buttons_x = 7;
		menu_buttons_y = 4;
		menu_button_width = 159;
		menu_button_height = 145;
		off_button_width = 38;
		off_button_height = 106;
		off_button_y_offset = 316;
		slideshow_button_width = 126;
		slideshow_button_height = 106;
		right_button_width = 164;
		right_button_height = 66;
		slider_x_offset = 242;
		slider_y_offset = 410;
		slider_width = 364;
		slider_height = 90;
		slider_x_raw_offset = 22;
		slider_y_raw_offset = 10;
		background_label_width = 1114;
		background_label_height = 800;
		digits_x_offset = 307;
		digits_y_offset = 281;
		digits_width = 46;
		digits_height = 72;
		digits_1_2_offset = 3;
		digits_2_3_offset = 27;
		video_x_offset = 0;
		video_y_offset = 117;
		video_width = 1114;
		video_height = 576;
	} else if(xResolution >= 1024 && yResolution >= 768 &&
		QDir(folder + "/1024x768/").exists())
	{
		xResolution = 1024;
		yResolution = 768;
		path = folder + "/1024x768/";
		header_height = 136;
		menu_buttons_x = 4;
		menu_buttons_y = 3;
		menu_button_width = 205;
		menu_button_height = 170;
		off_button_width = 39;
		off_button_height = 120;
		off_button_y_offset = 172;
		slideshow_button_width = 163;
		slideshow_button_height = 120;
		right_button_width = 204;
		right_button_height = 85;
		slider_x_offset = 172;
		slider_y_offset = 440;
		slider_width = 467;
		slider_height = 115;
		slider_x_raw_offset = 29;
		slider_y_raw_offset = 13;
		background_label_width = 820;
		background_label_height = 768;
		digits_x_offset = 307;
		digits_y_offset = 215;
		digits_width = 46;
		digits_height = 72;
		digits_1_2_offset = 3;
		digits_2_3_offset = 27;
		video_x_offset = 0;
		video_y_offset = 154;
		video_width = 819;
		video_height = 472;
	} else if(xResolution >= 800 && yResolution >= 600 &&
		QDir(folder + "/800x600/").exists())
	{
		xResolution = 800;
		yResolution = 600;
		path = folder + "/800x600/";
		header_height = 106;
		menu_buttons_x = 4;
		menu_buttons_y = 3;
		menu_button_width = 160;
		menu_button_height = 134;
		off_button_width = 30;
		off_button_height = 92;
		off_button_y_offset = 133;
		slideshow_button_width = 127;
		slideshow_button_height = 92;
		right_button_width = 160;
		right_button_height = 67;
		slider_x_offset = 135;
		slider_y_offset = 344;
		slider_width = 364;
		slider_height = 90;
		slider_x_raw_offset = 22;
		slider_y_raw_offset = 10;
		background_label_width = 640;
		background_label_height = 600;
		digits_x_offset = 195;
		digits_y_offset = 131;
		digits_width = 46;
		digits_height = 72;
		digits_1_2_offset = 3;
		digits_2_3_offset = 27;
		video_x_offset = 0;
		video_y_offset = 129;
		video_width = 640;
		video_height = 352;
	} else if(xResolution >= 800 && yResolution >= 480 &&
		QDir(folder + "/800x480/").exists())
	{
		xResolution = 800;
		yResolution = 480;
		path = folder + "/800x480/";
		header_height = 105;
		menu_buttons_x = 4;
		menu_buttons_y = 2;
		menu_button_width = 160;
		menu_button_height = 134;
		off_button_width = 33;
		off_button_height = 106;
		slideshow_button_width = 127;
		slideshow_button_height = 106;
		right_button_width = 160;
		right_button_height = 67;
		slider_x_offset = 137;
		slider_y_offset = 268;
		slider_width = 364;
		slider_height = 90;
		slider_x_raw_offset = 22;
		slider_y_raw_offset = 10;
		background_label_width = 640;
		background_label_height = 480;
		digits_x_offset = 196;
		digits_y_offset = 130;
		digits_width = 46;
		digits_height = 72;
		digits_1_2_offset = 3;
		digits_2_3_offset = 25;
		video_x_offset = 0;
		video_y_offset = 108;
		video_width = 640;
		video_height = 320;
	} else if(xResolution >= 640 && yResolution >= 480 && 
		QDir(folder + "/640x480/").exists())
	{
		xResolution = 640;
		yResolution = 480;
		path = folder + "/640x480/";
		header_height = 106;
		menu_buttons_x = 3;
		menu_buttons_y = 2;
		menu_button_width = 160;
		menu_button_height = 134;
		off_button_width = 33;
		off_button_height = 106;
		slideshow_button_width = 127;
		slideshow_button_height = 106;
		right_button_width = 160;
		right_button_height = 67;
		slider_x_offset = 53;
		slider_y_offset = 266;
		slider_width = 364;
		slider_height = 90;
		slider_x_raw_offset = 22;
		slider_y_raw_offset = 10;
		background_label_width = 478;
		background_label_height = 480;
		digits_x_offset = 128;
		digits_y_offset = 130;
		digits_width = 46;
		digits_height = 72;
		digits_1_2_offset = 3;
		digits_2_3_offset = 27;
		video_x_offset = 0;
		video_y_offset = 108;
		video_width = 480;
		video_height = 264;
	} else if(xResolution >= 480 && yResolution >= 272 && 
		QDir(folder + "/480x272/").exists())
	{
		xResolution = 480;
		yResolution = 272;
		path = folder + "/480x272/";
		header_height = 36;
		menu_buttons_x = 3;
		menu_buttons_y = 2;
		menu_button_width = 120;
		menu_button_height = 100;
		off_button_width = 30;
		off_button_height = 36;
		slideshow_button_width = 90;
		slideshow_button_height = 36;
		right_button_width = 120;
		right_button_height = 50;
		slider_x_offset = 36;
		slider_y_offset = 148;
		slider_width = 284;
		slider_height = 66;
		slider_x_raw_offset = 17;
		slider_y_raw_offset = 8;
		background_label_width = 358;
		background_label_height = 272;
		digits_x_offset = 104;
		digits_y_offset = 50;
		digits_width = 28;
		digits_height = 46;
		digits_1_2_offset = 2;
		digits_2_3_offset = 15;
		video_x_offset = 2;
		video_y_offset = 38;
		video_width = 356;
		video_height = 196;
	} else {
		qCritical() << "ERROR: Could not find a subfolder of" << folder << "matching the screen resolution";
		exit(1);
	}
	qDebug() << "Loading images from" << path;

    if (MainView->objectName().isEmpty()) {
        MainView->setObjectName("MainView");
	}

	MainView->setGeometry(QRect(0, 0, xResolution, yResolution));
    MainView->setAutoFillBackground(false);
    label_background = new QLabel(MainView);
    label_background->setObjectName("label_background");
    label_background->setGeometry(QRect(0, 0, xResolution, yResolution));
    label_background->setPixmap(QPixmap(path + "buttons_all_0.jpg"));
    label_background->setScaledContents(true);
    label_background->setAlignment(Qt::AlignBottom|Qt::AlignLeading|Qt::AlignLeft);

	buttons_off = new QPixmap(path + "buttons_all_0.jpg", 0, Qt::AutoColor);
	buttons_on = new QPixmap(path + "buttons_all_1.jpg", 0, Qt::AutoColor);

    menuButtonGroup = new QButtonGroup(MainView);
    menuButtonGroup->setObjectName("menuButtonGroup");

	for (i=0; i<menu_buttons_x * menu_buttons_y; i++)
	{
		unsigned int x = i % menu_buttons_x;
		unsigned int y = i / menu_buttons_x;
		QIcon new_icon;
		QPushButton *new_button = new QPushButton(MainView);

		new_icon.addPixmap(buttons_off->copy(x*menu_button_width, header_height + y*menu_button_height,
							menu_button_width, menu_button_height), QIcon::Normal, QIcon::Off);
		new_icon.addPixmap(buttons_on->copy(x*menu_button_width, header_height + y*menu_button_height,
							menu_button_width, menu_button_height), QIcon::Normal, QIcon::On);
		icons.append(new_icon);

		new_button->setGeometry(QRect(x*menu_button_width, header_height + y*menu_button_height,
										menu_button_width, menu_button_height));
		new_button->setIcon(icons.last());
		new_button->setIconSize(QSize(menu_button_width, menu_button_height));
		new_button->setCheckable(true);
		new_button->setAutoExclusive(true);
		buttons.append(new_button);
		menuButtonGroup->addButton(buttons.last());
	}

    rightButtonGroup = new QButtonGroup(MainView);
    rightButtonGroup->setObjectName("rightButtonGroup");
    button_off = new QPushButton(MainView);
    button_off->setObjectName("button_off");
    button_off->setGeometry(QRect(menu_buttons_x * menu_button_width + slideshow_button_width, header_height + 
								4 * right_button_height + off_button_y_offset, off_button_width, off_button_height));
	icon_off.addPixmap(buttons_off->copy(menu_buttons_x * menu_button_width + slideshow_button_width, header_height + 
								4 * right_button_height + off_button_y_offset, off_button_width, off_button_height), QIcon::Normal, QIcon::Off);
	icon_off.addPixmap(buttons_on->copy(menu_buttons_x * menu_button_width + slideshow_button_width, header_height + 
								4 * right_button_height + off_button_y_offset, off_button_width, off_button_height), QIcon::Normal, QIcon::On);
    button_off->setIcon(icon_off);
    button_off->setIconSize(QSize(off_button_width, off_button_height));
    button_off->setCheckable(true);
    button_off->setAutoExclusive(false);
	button_slideshow = new QPushButton(MainView);
	button_slideshow->setObjectName("button_slideshow");
	button_slideshow->setGeometry(QRect(menu_buttons_x * menu_button_width, header_height + 4 * right_button_height +
								off_button_y_offset, slideshow_button_width, slideshow_button_height));
	icon_slideshow.addPixmap(buttons_off->copy(menu_buttons_x * menu_button_width, header_height + 4 * right_button_height +
								off_button_y_offset, slideshow_button_width, slideshow_button_height), QIcon::Normal, QIcon::Off);
	icon_slideshow.addPixmap(buttons_on->copy(menu_buttons_x * menu_button_width, header_height + 4 * right_button_height +
								off_button_y_offset, slideshow_button_width, slideshow_button_height), QIcon::Normal, QIcon::On);
	button_slideshow->setIcon(icon_slideshow);
	button_slideshow->setIconSize(QSize(slideshow_button_width, slideshow_button_height));
	button_slideshow->setCheckable(true);
	button_slideshow->setAutoExclusive(false);
    button_video = new QPushButton(MainView);
    rightButtonGroup->addButton(button_video);
    button_video->setObjectName("button_video");
    button_video->setGeometry(QRect(menu_buttons_x * menu_button_width, header_height,
								right_button_width, right_button_height));
	icon_video.addPixmap(buttons_off->copy(menu_buttons_x * menu_button_width, header_height,
								right_button_width, right_button_height), QIcon::Normal, QIcon::Off);
	icon_video.addPixmap(buttons_on->copy(menu_buttons_x * menu_button_width, header_height,
								right_button_width, right_button_height), QIcon::Normal, QIcon::On);
    button_video->setIcon(icon_video);
    button_video->setIconSize(QSize(right_button_width, right_button_height));
    button_video->setCheckable(true);
    button_video->setAutoExclusive(true);
    button_menu = new QPushButton(MainView);
    rightButtonGroup->addButton(button_menu);
    button_menu->setObjectName("button_menu");
    button_menu->setGeometry(QRect(menu_buttons_x * menu_button_width, header_height + right_button_height,
								right_button_width, right_button_height));
	icon_menu.addPixmap(buttons_off->copy(menu_buttons_x * menu_button_width, header_height + right_button_height,
								right_button_width, right_button_height), QIcon::Normal, QIcon::Off);
	icon_menu.addPixmap(buttons_on->copy(menu_buttons_x * menu_button_width, header_height + right_button_height,
								right_button_width, right_button_height), QIcon::Normal, QIcon::On);
    button_menu->setIcon(icon_menu);
    button_menu->setIconSize(QSize(right_button_width, right_button_height));
    button_menu->setCheckable(true);
    button_menu->setAutoExclusive(true);
    button_slider = new QPushButton(MainView);
    rightButtonGroup->addButton(button_slider);
    button_slider->setObjectName("button_slider");
    button_slider->setGeometry(QRect(menu_buttons_x * menu_button_width, header_height + 2 * right_button_height,
								right_button_width, right_button_height));
	icon_slider.addPixmap(buttons_off->copy(menu_buttons_x * menu_button_width, header_height + 2 * right_button_height,
								right_button_width, right_button_height), QIcon::Normal, QIcon::Off);
	icon_slider.addPixmap(buttons_on->copy(menu_buttons_x * menu_button_width, header_height + 2 * right_button_height,
								right_button_width, right_button_height), QIcon::Normal, QIcon::On);
    button_slider->setIcon(icon_slider);
    button_slider->setIconSize(QSize(right_button_width, right_button_height));
    button_slider->setCheckable(true);
    button_slider->setAutoExclusive(true);
    button_help = new QPushButton(MainView);
    rightButtonGroup->addButton(button_help);
    button_help->setObjectName("button_help");
    button_help->setGeometry(QRect(menu_buttons_x * menu_button_width, header_height + 3 * right_button_height,
								right_button_width, right_button_height));
	icon_help.addPixmap(buttons_off->copy(menu_buttons_x * menu_button_width, header_height + 3 * right_button_height,
								right_button_width, right_button_height), QIcon::Normal, QIcon::Off);
	icon_help.addPixmap(buttons_on->copy(menu_buttons_x * menu_button_width, header_height + 3 * right_button_height,
								right_button_width, right_button_height), QIcon::Normal, QIcon::On);
    button_help->setIcon(icon_help);
    button_help->setIconSize(QSize(right_button_width, right_button_height));
    button_help->setCheckable(true);
    button_help->setAutoExclusive(true);
    label_slider = new QLabel(MainView);
    label_slider->setObjectName("label_slider");
    label_slider->setGeometry(QRect(0, 0, background_label_width, background_label_height));
    label_slider->setPixmap(QPixmap(path + "background_slider.jpg"));
    label_slider->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
	label_slider->hide();
    label_help = new QLabel(MainView);
	label_help->hide();
    label_help->setObjectName("label_help");
    label_help->setGeometry(QRect(0, 0, background_label_width, background_label_height));
    label_help->setPixmap(QPixmap(path + "background_help.jpg"));
    label_help->setScaledContents(true);
    label_help->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
    label_menu = new QLabel(MainView);
    label_menu->setObjectName("label_menu");
    label_menu->setGeometry(QRect(0, 0, background_label_width, background_label_height));
    label_menu->setPixmap(QPixmap(path + "background_menu.jpg"));
    label_menu->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
	label_menu->hide();
#ifdef BACKLIGHT
	qbacklight = new QBacklight();
	initial_backlight = qbacklight->getBrightness();
	if (initial_backlight == 100) {
		initial_digit1 = 9;
		initial_digit2 = 9;
		initial_digit3 = 9;
	} else {
		initial_digit1 = initial_backlight / 10;
		initial_digit2 = initial_backlight % 10;
	}
	
#endif
	qgufslider = new QGufSlider(this, 16, initial_backlight,
					QPixmap(path + "bar_0.jpg").copy(slider_x_raw_offset,
							slider_y_raw_offset, slider_width, slider_height),
					QPixmap(path + "bar_16.jpg").copy(slider_x_raw_offset,
							slider_y_raw_offset, slider_width, slider_height));
    qgufslider->hide();
    qgufslider->setGeometry(QRect(slider_x_offset, slider_y_offset, slider_width, slider_height));
	qdigit1 = new QDigit(this, initial_digit1, QPixmap(path + "digits_2.jpg"));
    qdigit2 = new QDigit(this, initial_digit2, QPixmap(path + "digits_1.jpg"));
    qdigit3 = new QDigit(this, initial_digit3, QPixmap(path + "digits_0.jpg"));
    qdigit1->hide();
    qdigit2->hide();
    qdigit3->hide();
	qdigit1->setGeometry(QRect(digits_x_offset, digits_y_offset, digits_width, digits_height));
    qdigit2->setGeometry(QRect(digits_x_offset + digits_width + digits_1_2_offset, digits_y_offset, digits_width, digits_height));
    qdigit3->setGeometry(QRect(digits_x_offset + 2 * digits_width + digits_2_3_offset, digits_y_offset, digits_width, digits_height));	
#ifdef VIDEO
	if (QFile(VIDEO_FILE).exists())
	{
		if(isMultiScreen) {
			video_width = (unsigned int)QApplication::desktop()->screenGeometry(1).width();
			video_height = (unsigned int)QApplication::desktop()->screenGeometry(1).height();
			videoView = new VideoView();
			qvideo = new QVideo(videoView, VIDEO_FILE);
			qvideo->setGeometry(QRect(0, 0, video_width, video_height));
			videoView->show();
		} else {
			qvideo = new QVideo(this, VIDEO_FILE);
			qvideo->hide();
			qvideo->setGeometry(QRect(video_x_offset, video_y_offset, video_width, video_height));
		}
					
	}
#endif
	qslideshow = new QSlideShow(this);
    qslideshow->hide();
	qslideshow->setGeometry(QRect(0, 0, xResolution, yResolution));
	i=1;
	while (QFile(path + "slide_" + QString::number(i)
				+ ".jpg").exists())
	{
		qslideshow->addImage(QPixmap(path + "slide_" + QString::number(i)
				+ ".jpg"));
		i++;
	}
    QMetaObject::connectSlotsByName(MainView);
	timer = new QTimer(this);
    timeline = new QTimeLine(T_BEGINN,this);
    timer->setInterval(PIC_DELAY);
    timeline->start();
	
    // ****** CONNECTIONS FOR VIDEO ******
    QObject::connect(button_video, SIGNAL(pressed()), this,SLOT(checkButton()));
    QObject::connect(button_video, SIGNAL(pressed()), label_menu,SLOT(show()));
    QObject::connect(button_slider, SIGNAL(pressed()), label_menu,SLOT(hide()));
    QObject::connect(button_menu, SIGNAL(pressed()), label_menu,SLOT(hide()));
    QObject::connect(button_help, SIGNAL(pressed()), label_menu,SLOT(hide()));
#ifdef VIDEO
	if (QFile(VIDEO_FILE).exists())
	{
		QObject::connect(button_video, SIGNAL(pressed()), qvideo,SLOT(play()));
		QObject::connect(button_video, SIGNAL(pressed()), qvideo,SLOT(show()));
		QObject::connect(button_slider, SIGNAL(pressed()), qvideo,SLOT(stop()));
		QObject::connect(button_slider, SIGNAL(pressed()), qvideo,SLOT(hide()));
		QObject::connect(button_menu, SIGNAL(pressed()), qvideo,SLOT(stop()));
		QObject::connect(button_menu, SIGNAL(pressed()), qvideo,SLOT(hide()));
		QObject::connect(button_help, SIGNAL(pressed()), qvideo,SLOT(stop()));
		QObject::connect(button_help, SIGNAL(pressed()), qvideo,SLOT(hide()));
	}
#endif
    // ****** CONNECTIONS FOR DIGIT & SLIDER ******
    QObject::connect(button_video, SIGNAL(pressed()), label_slider, SLOT(hide()));
    QObject::connect(button_video, SIGNAL(pressed()), qgufslider,SLOT(hide()));
    QObject::connect(button_video, SIGNAL(pressed()), qdigit1,SLOT(hide()));
    QObject::connect(button_video, SIGNAL(pressed()), qdigit2,SLOT(hide()));
    QObject::connect(button_video, SIGNAL(pressed()), qdigit3,SLOT(hide()));
    QObject::connect(button_slider, SIGNAL(pressed()), this,SLOT(checkButton()));
    QObject::connect(button_slider, SIGNAL(pressed()), label_slider, SLOT(show()));
    QObject::connect(button_slider, SIGNAL(pressed()), qgufslider,SLOT(show()));
    QObject::connect(button_slider, SIGNAL(pressed()), qdigit1,SLOT(show()));
    QObject::connect(button_slider, SIGNAL(pressed()), qdigit2,SLOT(show()));
    QObject::connect(button_slider, SIGNAL(pressed()), qdigit3,SLOT(show()));
    QObject::connect(button_menu, SIGNAL(pressed()), this,SLOT(checkButton()));
    QObject::connect(button_menu, SIGNAL(pressed()), label_slider, SLOT(hide()));
    QObject::connect(button_menu, SIGNAL(pressed()), qgufslider,SLOT(hide()));
    QObject::connect(button_menu, SIGNAL(pressed()), qdigit1,SLOT(hide()));
    QObject::connect(button_menu, SIGNAL(pressed()), qdigit2,SLOT(hide()));
    QObject::connect(button_menu, SIGNAL(pressed()), qdigit3,SLOT(hide()));
    QObject::connect(button_help, SIGNAL(pressed()), this,SLOT(checkButton()));
    QObject::connect(button_help, SIGNAL(pressed()), label_slider, SLOT(hide()));
    QObject::connect(button_help, SIGNAL(pressed()), qgufslider,SLOT(hide()));
    QObject::connect(button_help, SIGNAL(pressed()), qdigit1,SLOT(hide()));
    QObject::connect(button_help, SIGNAL(pressed()), qdigit2,SLOT(hide()));
    QObject::connect(button_help, SIGNAL(pressed()), qdigit3,SLOT(hide()));
    QObject::connect(qgufslider, SIGNAL(pos1(unsigned int)), qdigit1, SLOT(setNum(unsigned int)));
    QObject::connect(qgufslider, SIGNAL(pos2(unsigned int)), qdigit2, SLOT(setNum(unsigned int)));
    QObject::connect(qgufslider, SIGNAL(pos3(unsigned int)), qdigit3, SLOT(setNum(unsigned int)));
    QObject::connect(qgufslider, SIGNAL(isActive()), this,SLOT(restartTL()));
#ifdef BACKLIGHT
	QObject::connect(qgufslider, SIGNAL(value(unsigned int)), qbacklight, SLOT(setBrightness(unsigned int)));
#endif

    // ****** CONNECTIONS FOR HELP ******
    QObject::connect(button_video, SIGNAL(pressed()), label_help,SLOT(hide()));
    QObject::connect(button_slider, SIGNAL(pressed()), label_help,SLOT(hide()));
    QObject::connect(button_menu, SIGNAL(pressed()), label_help,SLOT(hide()));
    QObject::connect(button_help, SIGNAL(pressed()), label_help,SLOT(show()));

    // ****** CONNECTIONS FOR TIMER CONTROL ******
    QObject::connect(timeline, SIGNAL(finished()), timer,SLOT(start()));
    QObject::connect(timeline, SIGNAL(finished()), qslideshow,SLOT(show()));
    QObject::connect(timeline, SIGNAL(finished()), qslideshow,SLOT(startSlide()));
    QObject::connect(timer, SIGNAL(timeout()), qslideshow,SLOT(nextPixmap()));

    QObject::connect(qslideshow, SIGNAL(finished()),qslideshow,SLOT(hide()));
    QObject::connect(qslideshow, SIGNAL(finished()),timer,SLOT(stop()));
    QObject::connect(qslideshow, SIGNAL(finished()),timeline,SLOT(start()));
    QObject::connect(qslideshow, SIGNAL(finished()), this, SLOT(uncheckSlideshowButton()));

    QObject::connect(button_slideshow, SIGNAL(pressed()), timer,SLOT(start()));

    // ****** BUTTONGROUP 2 ******
	foreach(QPushButton *button, buttons) {
		QObject::connect(button, SIGNAL(pressed()), this,SLOT(restartTL()));
	    QObject::connect(button, SIGNAL(pressed()), this,SLOT(checkButton()));
	}

    // ****** BUTTONGROUP 1 ******
    QObject::connect(button_menu, SIGNAL(pressed()), this,SLOT(restartTL()));
    QObject::connect(button_slider, SIGNAL(pressed()), this,SLOT(restartTL()));
    QObject::connect(button_help, SIGNAL(pressed()), this,SLOT(restartTL()));
	if (QFile(VIDEO_FILE).exists())
	{
		QObject::connect(button_video, SIGNAL(pressed()), timeline,SLOT(stop()));
		QObject::connect(button_menu, SIGNAL(pressed()), this,SLOT(restartTL()));
		QObject::connect(button_slider, SIGNAL(pressed()), this,SLOT(restartTL()));
		QObject::connect(button_help, SIGNAL(pressed()), this,SLOT(restartTL()));
	} else {
		QObject::connect(button_video, SIGNAL(pressed()), this,SLOT(restartTL()));
	}

    // ***** QUIT BUTTON*****
    QObject::connect(button_off,SIGNAL(pressed()), this,SLOT(rebootSystem()));
    QObject::connect(button_slideshow, SIGNAL(pressed()), qslideshow,SLOT(show()));
    QObject::connect(button_slideshow, SIGNAL(pressed()), qslideshow,SLOT(startSlide()));
    QObject::connect(button_slideshow, SIGNAL(pressed()), timeline,SLOT(stop()));
    QObject::connect(button_slideshow, SIGNAL(pressed()), qslideshow,SLOT(raise()));
}
   
