#include <stdlib.h>
#include <stdio.h>
#include <linux/serial.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <termios.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <time.h>

#define LENGTH 10
#define error_message(...) printf(__VA_ARGS__)

static int
set_interface_attribs (int fd, int speed, int parity)
{
        struct termios tty;
        memset (&tty, 0, sizeof tty);
        if (tcgetattr (fd, &tty) != 0)
        {
                error_message ("error %d from tcgetattr", errno);
                return -1;
        }

        cfsetospeed (&tty, speed);
        cfsetispeed (&tty, speed);

        tty.c_cflag = (tty.c_cflag & ~CSIZE) | CS8;     // 8-bit chars
        // disable IGNBRK for mismatched speed tests; otherwise receive break
        // as \000 chars
        tty.c_iflag &= ~IGNBRK;         // disable break processing
        tty.c_lflag = 0;                // no signaling chars, no echo,
                                        // no canonical processing
        tty.c_oflag = 0;                // no remapping, no delays
        tty.c_cc[VMIN]  = 0;            // read doesn't block
        tty.c_cc[VTIME] = 5;            // 0.5 seconds read timeout

        tty.c_iflag &= ~(IXON | IXOFF | IXANY); // shut off xon/xoff ctrl

        tty.c_cflag |= (CLOCAL | CREAD);// ignore modem controls,
                                        // enable reading
        tty.c_cflag &= ~(PARENB | PARODD);      // shut off parity
        tty.c_cflag |= parity;
        tty.c_cflag &= ~CSTOPB;
        tty.c_cflag &= ~CRTSCTS;

        if (tcsetattr (fd, TCSANOW, &tty) != 0)
        {
                error_message ("error %d from tcsetattr", errno);
                return -1;
        }
        return 0;
}

static void
set_blocking (int fd, int should_block)
{
        struct termios tty;
        memset (&tty, 0, sizeof tty);
        if (tcgetattr (fd, &tty) != 0)
        {
                error_message ("error %d from tggetattr", errno);
                return;
        }

        tty.c_cc[VMIN]  = should_block ? 1 : 0;
        tty.c_cc[VTIME] = 5;            // 0.5 seconds read timeout

        if (tcsetattr (fd, TCSANOW, &tty) != 0)
                error_message ("error %d setting term attributes", errno);
}

int main(int argc, char* argv[])
{
	int fd;
	int ret;
	char buf[LENGTH];
	size_t bufsize = LENGTH;
	int i, written;

	for( i = 0; i < bufsize; i++)
		buf[i] = 'a' + i;


	printf("port open\r\n");

	fd =  open("/dev/ttymxc2", O_RDWR|O_NOCTTY);
	if(fd==-1) 
	{
		printf("couldn't open port\r\n");
		return 0;
	}
	
	printf("reset rs485\r\n");
	{
		struct serial_rs485 rs485conf;
		rs485conf.flags = 0;
		rs485conf.delay_rts_before_send = 0;
		rs485conf.delay_rts_after_send = 0;
		ret = ioctl(fd, TIOCSRS485, &rs485conf);
		printf("reset rs485 returned %d\r\n", ret);
	}

	printf("set rs485\r\n");
	{
		struct serial_rs485 rs485conf;
		rs485conf.flags = SER_RS485_ENABLED | SER_RS485_RTS_ON_SEND;
		rs485conf.delay_rts_before_send = 4;
		rs485conf.delay_rts_after_send = 0;
		ret = ioctl(fd, TIOCSRS485, &rs485conf);
		printf("set rs485 returned %d\r\n", ret);
	}


	//set_interface_attribs (fd, B115200, 0);  // set speed to 115,200 bps, 8n1 (no parity)
	set_interface_attribs (fd, B300, 0);  // set speed to 115,200 bps, 8n1 (no parity)
	//set_blocking (fd, 1);                // set blocking

	printf("done\r\n");

	written = 0;
	for( i=0; i < 10;)
	{
		ret = write(fd, & buf[written], bufsize -written );
		if( ret < 0)
		{
			printf("Error in write: %d : %s\n", errno, strerror(errno));
			break;
		}
		written += ret;
		if( written == bufsize)
		{
			written = 0;
			i++;

			printf("%d \n", i);
			sleep(1);
			continue;
		}	
	}
	close(fd);
	return 0;
}

