#include <time.h>
#include <stdio.h>
#include <stdbool.h>
#include <string.h>
#include <errno.h>
#include <termios.h>
#include <unistd.h>
#include <fcntl.h>

#define error_message(...) printf(__VA_ARGS__)

static struct timespec diff(struct timespec start, struct timespec end)
{
	struct timespec temp;
	if ((end.tv_nsec-start.tv_nsec)<0) {
		temp.tv_sec = end.tv_sec-start.tv_sec-1;
		temp.tv_nsec = 1000000000+end.tv_nsec-start.tv_nsec;
	} else {
		temp.tv_sec = end.tv_sec-start.tv_sec;
		temp.tv_nsec = end.tv_nsec-start.tv_nsec;
	}
	return temp;
}

static unsigned int diff_ns(struct timespec start, struct timespec end)
{
	struct timespec d = diff( start, end);

	return d.tv_nsec + SEC(d.tv_sec );
}



static struct timespec add(struct timespec a, struct timespec b)
{
	struct timespec temp = a;

	temp.tv_nsec += b.tv_nsec;
	temp.tv_sec += b.tv_sec;
	if( temp.tv_nsec >= SEC(1))
	{
		temp.tv_nsec -= SEC(1);
		temp.tv_sec += 1;
	}
	return temp;
}

static int
set_interface_attribs (int fd, int speed, int parity)
{
        struct termios tty;
        memset (&tty, 0, sizeof tty);
        if (tcgetattr (fd, &tty) != 0)
        {
                error_message ("error %d from tcgetattr", errno);
                return -1;
        }

        cfsetospeed (&tty, speed);
        cfsetispeed (&tty, speed);

        tty.c_cflag = (tty.c_cflag & ~CSIZE) | CS8;     // 8-bit chars
        // disable IGNBRK for mismatched speed tests; otherwise receive break
        // as \000 chars
        tty.c_iflag &= ~IGNBRK;         // disable break processing
        tty.c_lflag = 0;                // no signaling chars, no echo,
                                        // no canonical processing
        tty.c_oflag = 0;                // no remapping, no delays
        tty.c_cc[VMIN]  = 0;            // read doesn't block
        tty.c_cc[VTIME] = 5;            // 0.5 seconds read timeout

        tty.c_iflag &= ~(IXON | IXOFF | IXANY); // shut off xon/xoff ctrl

        tty.c_cflag |= (CLOCAL | CREAD);// ignore modem controls,
                                        // enable reading
        tty.c_cflag &= ~(PARENB | PARODD);      // shut off parity
        tty.c_cflag |= parity;
        tty.c_cflag &= ~CSTOPB;
        tty.c_cflag &= ~CRTSCTS;

        if (tcsetattr (fd, TCSANOW, &tty) != 0)
        {
                error_message ("error %d from tcsetattr", errno);
                return -1;
        }
        return 0;
}

static void
set_blocking (int fd, int should_block)
{
        struct termios tty;
        memset (&tty, 0, sizeof tty);
        if (tcgetattr (fd, &tty) != 0)
        {
                error_message ("error %d from tggetattr", errno);
                return;
        }

        tty.c_cc[VMIN]  = should_block ? 1 : 0;
        tty.c_cc[VTIME] = 5;            // 0.5 seconds read timeout

        if (tcsetattr (fd, TCSANOW, &tty) != 0)
                error_message ("error %d setting term attributes", errno);
}

static int fill_buffer(char * buf, int len)
{
	FILE *fp;
	int read;
	fp = fopen("/dev/urandom", "r");
	if( !fp )
	{
		return errno;
	}
	read = fread(buf, 1, len, fp);
	if( read != len)
	{
		return errno;
	}
	fclose(fp);
	return 0;
}

static void
create_data( int no, const char * inbuf, int inbuflen, char * buf, int len)
{
    int i;
    for(i = 0; i < len/2; i ++)
    {
        buf[i] = inbuf[ (i + 20 * no) % inbuflen ];
        buf[i + len/2] = inbuf[ (i + 20 * no) % inbuflen ];
    }
}
static int
verify_data( char * buf, int len)
{
    int i;
    for(i = 0; i < len/2; i ++)
    {
        if( buf[i] != buf[i + len/2])
            return -1;
    }
    return 0;
}

#define COUNT 100
#define BYTES 78
int main ( int argc, char *argv[]) 
{
	char buff[BYTES*10];
	int error;
	int fd;
	char *portname = "/dev/ttymxc1";
	char buf [BYTES];
	int i, n;
	struct timespec t1, t2;
    unsigned int res = 0;
    bool sender = false;
    struct timespec duration, period, last_t2;
    unsigned int period_sum = 0;
    unsigned int duration_sum = 0;
    unsigned int period_max = 0;
    unsigned int duration_max = 0;
    unsigned int tmp;


    if( argc > 1)
        sender = true;

    printf("Starting serialtest\n");
	int count = COUNT , bytes = BYTES;

    if(sender)
    {
        fill_buffer( buff, BYTES*10);
    }

	fd = open (portname, O_RDWR | O_NOCTTY | O_SYNC);
	if (fd < 0)
	{
			error_message ("error %d opening %s: %s", errno, portname, strerror (errno));
			return;
	}

    for( i = 0; i < BYTES; i++)
        buf[i] = 'a';

	set_interface_attribs (fd, B115200, 0);  // set speed to 115,200 bps, 8n1 (no parity)
	set_blocking (fd, 1);                // set blocking


    void get_timing( struct timespec t1, struct timespec t2)
    {
        duration = diff( t1, t2);
        tmp = duration.tv_nsec / 1000 ;
        duration_sum += tmp;
        if(tmp > duration_max) duration_max = tmp;
        period = diff( last_t2, t2 );
        tmp = period.tv_nsec / 1000;
        period_sum += tmp;
        if(tmp > period_max) period_max = tmp;

        last_t2 = t2;
    }

    period_sum = 0;
    duration_sum = 0;
    period_max = 0;
    duration_max = 0;

    while(true)
    {
    period_sum = 0;
    duration_sum = 0;
    period_max = 0;
    duration_max = 0;
    if(sender)
    {
        printf("Sending data ...\n");
        create_data( i, buff, BYTES * 10, buf, bytes);
        n = write (fd, buf, bytes);  // 
        clock_gettime( CLOCK_MONOTONIC, & last_t2);
        usleep (9900); 

        for( i = 0; i < count; i++)
        {
            create_data( i, buff, BYTES * 10, buf, bytes);
            clock_gettime( CLOCK_MONOTONIC, & t1);

            n = write (fd, buf, bytes);  // 

            clock_gettime( CLOCK_MONOTONIC, & t2);
            get_timing(t1,t2);
            if(n != bytes) res += 1;
            usleep (9900); 
        }

    }
    else{
        int cnt = 0;
        printf("Wait for data ...\n");
        do{
            n = read (fd, &buf[cnt], bytes - cnt);  // read up to 100 characters if ready to read
            cnt += n;
        }while(cnt < bytes);
        clock_gettime( CLOCK_MONOTONIC, & last_t2);
        usleep (9500); 

        for( i = 0; i < count; i++)
        {
            clock_gettime( CLOCK_MONOTONIC, & t1);

            do{
                n = read (fd, &buf[cnt], bytes - cnt);  // read up to 100 characters if ready to read
                cnt += n;
            }while(cnt < bytes);

            clock_gettime( CLOCK_MONOTONIC, & t2);
            
            get_timing(t1,t2);

            res += verify_data( buf, bytes);


            usleep (9500); 
        }
    }

	// print results
	printf("%s %d x %d bytes, Period avg %u us max %u us Duration avg %u us Duration max %u us ", 
        sender?"Send ":"Received", count, bytes, 
        period_sum / count, period_max, duration_sum / count, duration_max);
    printf(" t1:  %03d.%09d t2: %03d.%09d last_t2: %03d.%09d\n", t1.tv_sec, t1.tv_nsec, t2.tv_sec, t2.tv_nsec, last_t2.tv_sec, last_t2.tv_nsec);
    }
	return 0;
}
