
#include "debug.h"
#include "evdev_helper.h"
#include "touch_report.h"
//
//============================================
// Debug
//============================================
const char * evdev_flag_names[12] = {				
	[0] = "KEYBOARD_EVENTS	",
	[1] = "BUTTON_EVENTS	",
	[2] = "RELATIVE_EVENTS	",
	[3] = "ABSOLUTE_EVENTS	",
	[4] = "TOUCHPAD		",
	[5] = "INITIALIZED	",
	[6] = "TOUCHSCREEN	",
	[7] = "CALIBRATED	",
	[8] = "TABLET		",
	[9] = "UNIGNORE_ABSOLUTE ",
	[10] = "UNIGNORE_RELATIVE ",
	[11] = "RELATIVE_MODE	",
};

const char * evdev_event_names[0x18] = {
	[0x00] = "EV_SYN",
	[0x01] = "EV_KEY",
	[0x02] = "EV_REL",
	[0x03] = "EV_ABS",
	[0x04] = "EV_MSC",
	[0x05] = "EV_SW",
	[0x11] = "EV_LED",
	[0x12] = "EV_SND",
	[0x14] = "EV_REP",
	[0x15] = "EV_FF",
	[0x16] = "EV_PWR",
	[0x17] = "EV_FF_STATUS",
};

const char * evdev_rel_names[REL_CNT] = {
	[0] = "REL_X"		,
	[1] = "REL_Y"		,
	[2] = "REL_Z"		,
	[3] = "REL_RX"	,
	[4] = "REL_RY"	,
	[5] = "REL_RZ"	,
	[6] = "REL_HWHEEL",
	[7] = "REL_DIAL"	,
	[8] = "REL_WHEEL"	,
	[9] = "REL_MISC"	,
};

const char * evdev_abs_names[ABS_CNT] = {
	[0x00] = "ABS_X",
	[0x01] = "ABS_Y",
	[0x02] = "ABS_Z",
	[0x03] = "ABS_RX",
	[0x04] = "ABS_RY",
	[0x05] = "ABS_RZ",
	[0x06] = "ABS_THROTTLE",
	[0x07] = "ABS_RUDDER",
	[0x08] = "ABS_WHEEL",
	[0x09] = "ABS_GAS",
	[0x0a] = "ABS_BRAKE",
	[0x10] = "ABS_HAT0X",
	[0x11] = "ABS_HAT0Y",
	[0x12] = "ABS_HAT1X",
	[0x13] = "ABS_HAT1Y",
	[0x14] = "ABS_HAT2X",
	[0x15] = "ABS_HAT2Y",
	[0x16] = "ABS_HAT3X",
	[0x17] = "ABS_HAT3Y",
	[0x18] = "ABS_PRESSURE",
	[0x19] = "ABS_DISTANCE",
	[0x1a] = "ABS_TILT_X",
	[0x1b] = "ABS_TILT_Y",
	[0x1c] = "ABS_TOOL_WIDTH",

	[0x20] = "ABS_VOLUME",

	[0x28] = "ABS_MISC",

	[0x2f] = "ABS_MT_SLOT",
	[0x30] = "ABS_MT_TOUCH_MAJOR",
	[0x31] = "ABS_MT_TOUCH_MINOR",
	[0x32] = "ABS_MT_WIDTH_MAJOR",
	[0x33] = "ABS_MT_WIDTH_MINOR",
	[0x34] = "ABS_MT_ORIENTATION",
	[0x35] = "ABS_MT_POSITION_X",
	[0x36] = "ABS_MT_POSITION_Y",
	[0x37] = "ABS_MT_TOOL_TYPE",
	[0x38] = "ABS_MT_BLOB_ID",
	[0x39] = "ABS_MT_TRACKING_ID",
	[0x3a] = "ABS_MT_PRESSURE",
	[0x3b] = "ABS_MT_DISTANCE",
	[0x3c] = "ABS_MT_TOOL_X",
	[0x3d] = "ABS_MT_TOOL_Y",
};

const char * event_type(const struct input_event * ev)
{
	if(ev->type < ARRAY_SIZE(evdev_event_names))
		return evdev_event_names[ev->type];
	return "";
}
const char * event_name(const struct input_event * ev)
{
	switch(ev->type)
	{
	case EV_ABS:
		if(ev->code < ARRAY_SIZE(evdev_abs_names))
			return evdev_abs_names[ev->code];
		break;
	case EV_REL:
		if(ev->code < ARRAY_SIZE(evdev_rel_names))
			return evdev_rel_names[ev->code];
		break;
	}
	return "";
}

void dump_event(const char * prefix, const struct input_event * ev)
{
	pr_dbg("%s ev: %-20s %-10s %d  \n", prefix, event_name(ev),event_type(ev), ev->value);
}

void dump_report(const struct touch_report * report)
{
	pr_dbg("% 5d % 5d % 2d % 2d ||% 5d % 5d % 2d % 2d % 4d|% 5d % 5d % 2d % 2d % 4d|% 5d % 5d % 2d % 2d % 4d|% 5d % 5d % 2d % 2d % 4d|% 5d % 5d % 2d % 2d % 4d|\n",
		report->single_touch->current.x == NOT_YET_WRITTEN? -1:  report->single_touch->current.x, 
		report->single_touch->current.y == NOT_YET_WRITTEN? -1:  report->single_touch->current.y, 
		report->single_touch->current.down == NOT_YET_WRITTEN?-1:report->single_touch->current.down,
		report->single_touch->current.p == NOT_YET_WRITTEN?-1:report->single_touch->current.p,

		report->mt_touch[0].current.x == NOT_YET_WRITTEN? -1:  report->mt_touch[0].current.x, 
		report->mt_touch[0].current.y == NOT_YET_WRITTEN? -1:  report->mt_touch[0].current.y, 
		report->mt_touch[0].current.down == NOT_YET_WRITTEN?-1:report->mt_touch[0].current.down,
		report->mt_touch[0].current.mt_slot == NOT_YET_WRITTEN? -1:  report->mt_touch[0].current.mt_slot, 
		report->mt_touch[0].current.mt_tracking_id == NOT_YET_WRITTEN? 0:  report->mt_touch[0].current.mt_tracking_id, 

		report->mt_touch[1].current.x == NOT_YET_WRITTEN? -1:  report->mt_touch[1].current.x, 
		report->mt_touch[1].current.y == NOT_YET_WRITTEN? -1:  report->mt_touch[1].current.y, 
		report->mt_touch[1].current.down == NOT_YET_WRITTEN?-1:report->mt_touch[0].current.down,
		report->mt_touch[1].current.mt_slot == NOT_YET_WRITTEN? -1:  report->mt_touch[1].current.mt_slot, 
		report->mt_touch[1].current.mt_tracking_id == NOT_YET_WRITTEN? 0:  report->mt_touch[1].current.mt_tracking_id, 

		report->mt_touch[2].current.x == NOT_YET_WRITTEN? -1:  report->mt_touch[2].current.x, 
		report->mt_touch[2].current.y == NOT_YET_WRITTEN? -1:  report->mt_touch[2].current.y, 
		report->mt_touch[2].current.down == NOT_YET_WRITTEN?-1:report->mt_touch[0].current.down,
		report->mt_touch[2].current.mt_slot == NOT_YET_WRITTEN? -1:  report->mt_touch[2].current.mt_slot, 
		report->mt_touch[2].current.mt_tracking_id == NOT_YET_WRITTEN? 0:  report->mt_touch[2].current.mt_tracking_id, 

		report->mt_touch[3].current.x == NOT_YET_WRITTEN? -1:  report->mt_touch[3].current.x, 
		report->mt_touch[3].current.y == NOT_YET_WRITTEN? -1:  report->mt_touch[3].current.y, 
		report->mt_touch[3].current.down == NOT_YET_WRITTEN?-1:report->mt_touch[0].current.down,
		report->mt_touch[3].current.mt_slot == NOT_YET_WRITTEN? -1:  report->mt_touch[3].current.mt_slot, 
		report->mt_touch[3].current.mt_tracking_id == NOT_YET_WRITTEN? 0:  report->mt_touch[3].current.mt_tracking_id, 

		report->mt_touch[4].current.x == NOT_YET_WRITTEN? -1:  report->mt_touch[4].current.x, 
		report->mt_touch[4].current.y == NOT_YET_WRITTEN? -1:  report->mt_touch[4].current.y, 
		report->mt_touch[4].current.down == NOT_YET_WRITTEN?-1:report->mt_touch[0].current.down,
		report->mt_touch[4].current.mt_slot == NOT_YET_WRITTEN? -1:  report->mt_touch[4].current.mt_slot, 
		report->mt_touch[4].current.mt_tracking_id == NOT_YET_WRITTEN? 0:  report->mt_touch[4].current.mt_tracking_id 

		);

	pr_dbg("% 5d % 5d % 2d % 2d ||% 5d % 5d % 2d % 2d % 4d|% 5d % 5d % 2d % 2d % 4d|% 5d % 5d % 2d % 2d % 4d|% 5d % 5d % 2d % 2d % 4d|% 5d % 5d % 2d % 2d % 4d|\n",
		report->single_touch->processed.x == NOT_YET_WRITTEN? -1:  report->single_touch->processed.x, 
		report->single_touch->processed.y == NOT_YET_WRITTEN? -1:  report->single_touch->processed.y, 
		report->single_touch->processed.down == NOT_YET_WRITTEN?-1:report->single_touch->processed.down,
		report->single_touch->processed.p == NOT_YET_WRITTEN?-1:report->single_touch->processed.p,

		report->mt_touch[0].processed.x == NOT_YET_WRITTEN? -1:  report->mt_touch[0].processed.x, 
		report->mt_touch[0].processed.y == NOT_YET_WRITTEN? -1:  report->mt_touch[0].processed.y, 
		report->mt_touch[0].processed.down == NOT_YET_WRITTEN?-1:report->mt_touch[0].processed.down,
		report->mt_touch[0].processed.mt_slot == NOT_YET_WRITTEN? -1:  report->mt_touch[0].processed.mt_slot, 
		report->mt_touch[0].processed.mt_tracking_id == NOT_YET_WRITTEN? 0:  report->mt_touch[0].processed.mt_tracking_id, 

		report->mt_touch[1].processed.x == NOT_YET_WRITTEN? -1:  report->mt_touch[1].processed.x, 
		report->mt_touch[1].processed.y == NOT_YET_WRITTEN? -1:  report->mt_touch[1].processed.y, 
		report->mt_touch[1].processed.down == NOT_YET_WRITTEN?-1:report->mt_touch[0].processed.down,
		report->mt_touch[1].processed.mt_slot == NOT_YET_WRITTEN? -1:  report->mt_touch[1].processed.mt_slot, 
		report->mt_touch[1].processed.mt_tracking_id == NOT_YET_WRITTEN? 0:  report->mt_touch[1].processed.mt_tracking_id, 

		report->mt_touch[2].processed.x == NOT_YET_WRITTEN? -1:  report->mt_touch[2].processed.x, 
		report->mt_touch[2].processed.y == NOT_YET_WRITTEN? -1:  report->mt_touch[2].processed.y, 
		report->mt_touch[2].processed.down == NOT_YET_WRITTEN?-1:report->mt_touch[0].processed.down,
		report->mt_touch[2].processed.mt_slot == NOT_YET_WRITTEN? -1:  report->mt_touch[2].processed.mt_slot, 
		report->mt_touch[2].processed.mt_tracking_id == NOT_YET_WRITTEN? 0:  report->mt_touch[2].processed.mt_tracking_id, 

		report->mt_touch[3].processed.x == NOT_YET_WRITTEN? -1:  report->mt_touch[3].processed.x, 
		report->mt_touch[3].processed.y == NOT_YET_WRITTEN? -1:  report->mt_touch[3].processed.y, 
		report->mt_touch[3].processed.down == NOT_YET_WRITTEN?-1:report->mt_touch[0].processed.down,
		report->mt_touch[3].processed.mt_slot == NOT_YET_WRITTEN? -1:  report->mt_touch[3].processed.mt_slot, 
		report->mt_touch[3].processed.mt_tracking_id == NOT_YET_WRITTEN? 0:  report->mt_touch[3].processed.mt_tracking_id, 

		report->mt_touch[4].processed.x == NOT_YET_WRITTEN? -1:  report->mt_touch[4].processed.x, 
		report->mt_touch[4].processed.y == NOT_YET_WRITTEN? -1:  report->mt_touch[4].processed.y, 
		report->mt_touch[4].processed.down == NOT_YET_WRITTEN?-1:report->mt_touch[0].processed.down,
		report->mt_touch[4].processed.mt_slot == NOT_YET_WRITTEN? -1:  report->mt_touch[4].processed.mt_slot, 
		report->mt_touch[4].processed.mt_tracking_id == NOT_YET_WRITTEN? 0:  report->mt_touch[4].processed.mt_tracking_id 

		);
}

