#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <lua.h>
#include <lauxlib.h>
#include <lualib.h>

#include <debug.h>

#include "lua_interface.h"

#include "touch_report.h"
#include "touch_point.h"
#include "config.h"

struct lua_script{
	lua_State * L;
};

//===========================================================
//
//===========================================================
static int process(lua_State * L, int slot, struct touch_point * tp)
{
	int result, i;
	int args[4];

	result = lua_getglobal(L, "filter_process");
	if( result != LUA_TFUNCTION){
		pr_err("Couldn't find function 'filter_process': %s\n",  lua_tostring(L, -1));
		return -1;
	}

	int p = tp->p;
	if( p == NOT_YET_WRITTEN) p = tp->down;

	lua_pushinteger(L, slot);
	lua_pushinteger(L, tp->x);
	lua_pushinteger(L, tp->y);
	lua_pushinteger(L, p );
	lua_pushinteger(L, tp->down);
	if (lua_pcall(L, 5, 4, 0))
      fprintf(stderr,"Failure in filter_process: %s\n", lua_tostring(L, -1));
	pr_dbg("%s: Result: ", __func__);
	for(i = 0; i < 4; i ++)	{
		if (!lua_isnumber(L, -4 + i))	
			fprintf(stderr,"Failure in retrieving result from filter_process: %s\n", lua_tostring(L, -1));
		else{
			args[i] = lua_tonumber(L, -4 + i);
			pr_dbg(" %d: %d ", i, args[i]);
		}
	}	
	pr_dbg("\n");

	tp->x = args[0];
	tp->y = args[1];
	if( tp->p != NOT_YET_WRITTEN)
		tp->p = args[2];
	tp->down = args[3];

}

int lua_filtering( struct lua_script * lua, struct touch_report * report)
{
	int i;

	if( report->single_touch->updated)
		process(lua->L, -1, & (report->single_touch->processed));
	for( i = 0; i < report->max_tp_cnt; i++)
		if( report->mt_touch[i].updated )
			process(lua->L, i, & ( report->mt_touch[i].processed));
	return 0;
}

void free_lua_script( struct lua_script * lua)
{
	pr_func_line();
	free(lua);
	pr_func_line();
}

struct lua_script * setup_lua_script( const char * script)
{
	struct lua_script * l;
	lua_State * L;
	int result;
	int i;
   
	l = malloc( sizeof(struct lua_script));
	if( !l)	{
		pr_err("Failed to allocate data for lua\n");
		goto out_error;
	}
	memset(l, 0, sizeof(struct lua_script));

	L = luaL_newstate ();
	if( !L){
		pr_err("Failed to load %s: %s!\n", script, lua_tostring(L, -1));
		goto out_error;
	}
	l->L = L;
	luaL_openlibs(L);

	//luaopen_sample_reader (L);

	result = luaL_loadfile(L, script);
	if (result){
		pr_err("Failed to load %s: %s!\n", script, lua_tostring(L, -1));
		goto free_lua;
	} 

	result = lua_pcall(L, 0, 0, 0);
	if (result){
		pr_err("lua_pcall failed in %s: %s!\n", script, lua_tostring(L, -1));
	} 

	// Make sure script contains the needed functions
	for( i = -1; i < MAX_MT_POINTS; i++)
	{
		result = lua_getglobal(L, "filter_init");
		if( result != LUA_TFUNCTION){
			pr_err("Couldn't find function 'filter_init' in %d: %s\n", script, lua_tostring(L, -1));
			goto free_lua;
		}
		lua_pushinteger(L, i);
		if (lua_pcall(L, 1, 0, 0))
			fprintf(stderr,"Failure in filter_init: %s\n", lua_tostring(L, -1));
	}

	// Make sure script contains the needed functions
	result = lua_getglobal(L, "filter_reset");
	if( result != LUA_TFUNCTION){
		pr_err("Couldn't find function 'filter_reset' in %d: %s\n", script, lua_tostring(L, -1));
		goto free_lua;
	}
	lua_pop ( L, 1);

	// Make sure script contains the needed functions
	result = lua_getglobal(L, "filter_process");
	if( result != LUA_TFUNCTION){
		pr_err("Couldn't find function 'filter_process' in %d: %s\n", script, lua_tostring(L, -1));
		goto free_lua;
	}
	lua_pop (L, 1);

	return l;

free_lua:
	free(l);
out_error:
	return NULL;
}


