
#include "touch_point.h"
#include "debug.h"
#include "string.h"
#include <linux/input.h>

void touch_point_clear( struct touch_point * p)
{
	p->x = NOT_YET_WRITTEN;
	p->y = NOT_YET_WRITTEN;
	p->p = NOT_YET_WRITTEN;
	p->down = NOT_YET_WRITTEN;
	p->mt_slot = NOT_YET_WRITTEN;
	p->mt_tracking_id = NOT_YET_WRITTEN;
}

bool touch_point_empty( struct touch_point * p)
{
	if(
		p->x != NOT_YET_WRITTEN|
		p->y != NOT_YET_WRITTEN|
		p->p != NOT_YET_WRITTEN|
		p->down != NOT_YET_WRITTEN|
		p->mt_slot != NOT_YET_WRITTEN|
		p->mt_tracking_id != NOT_YET_WRITTEN
	) return false;
	return true;
}

int touch_point_container_add_touch_point( struct touch_point_container * c, struct touch_point * p)
{
	c->updated = true;
	memcpy( &c->previous, &c->current, sizeof(struct touch_point));
	memcpy( &c->current, p, sizeof(struct touch_point));
	//pr_dbg("Copy Down %d %d\n",c->current.down, c->previous.down);
}

int touch_point_add_event(struct touch_point * p, const struct input_event * ev, int slot )
{
	if( ev->type != EV_ABS){
		pr_err("%s: Wrong event type: %s\n", __func__, event_type(ev));
		return -1;
	}
	//pr_dbg("slot: %d  ", slot);
	dump_event("add_", ev);
	switch( ev->code )
	{
	case ABS_MT_POSITION_X:
		p->x = ev->value;
		break;
	case ABS_MT_POSITION_Y:
		p->y = ev->value;
		break;
	case ABS_MT_PRESSURE:
		p->p = ev->value;
		break;
	case ABS_MT_TRACKING_ID:
		if(ev->value == -1)
			p->down = 0;
		else
			p->mt_tracking_id = ev->value;
		break;
	default:
		pr_err("%s: Wrong event code: %s\n", __func__, event_name(ev));
		return -1;
	}
	if(p->down == NOT_YET_WRITTEN)
		p->down = 1;
	p->mt_slot = slot;
	p->time = ev->time;
	return 0;
}
