
#include "stdlib.h"
#include "string.h"

#include "tslib_wrapper.h"
#include "debug.h"
#include "config.h"
#include "touch_report.h"

#include "tslib/src/tslib-private.h"
#include "tslib/src/tslib-filter.h"

struct ts_slot
{
	struct tsdev ts;
	struct ts_sample s;
};

struct ts_wrapper
{
	int slot_count;
	struct ts_slot * slot;
};

static int tswrap_read(struct tslib_module_info *inf, struct ts_sample * s, int nr)
{
	struct ts_slot * slot = ( struct ts_slot * ) inf->dev->fd;
	memcpy(s,  &slot->s, sizeof(struct ts_sample));
	//pr_dbg("%s %d -- x: %d y:%d p:%d\n", __func__, __LINE__, s->x, s->y, s->pressure);
	return 1;
}

static int tswrap_fini(struct tslib_module_info *inf )
{
	free(inf);
	return 0;
}
static const struct tslib_ops tswrap_ops = {
	.read	= tswrap_read,
	.fini	= tswrap_fini,
};

struct ts_wrapper *  tswrap_initilize(int slots)
{
	int result, i;
	struct ts_wrapper * tswrap;

	pr_func_line();
	tswrap = malloc( sizeof(struct ts_wrapper));
	if( !tswrap){
		pr_err(" Failed to allocate memory for tswrapper\n");
		goto error;
	}
	memset(tswrap, 0, sizeof(struct ts_wrapper));
	tswrap->slot_count = slots;
	
	tswrap->slot = malloc( sizeof( struct ts_slot) * tswrap->slot_count );
	if( !tswrap->slot){
		pr_err(" Failed to allocate memory for ts_slot\n");
		goto error_free_tswrap;
	}
	

	for( i = 0; i < tswrap->slot_count; i++)
	{
		memset(& tswrap->slot[i].ts, 0, sizeof(struct tsdev));
		tswrap->slot[i].ts.fd = (int) &tswrap->slot[i];
		result = ts_config(&(tswrap->slot[i].ts) );
		if( result)
		{
			pr_err(" Failed to load tslib modules\n");
			// TODO cleanup aleady setup modules
			goto error_free;
		}
	}

	pr_dbg("ts_load_module: %d\n", result);

	return tswrap;

error_free:
error_free_slot:
	free(tswrap->slot);
error_free_tswrap:
	free(tswrap);
error:
	return NULL;
}

static void tswarp_free_tsdev(struct tsdev * ts)
{
	void *handle;
	int ret;
	struct tslib_module_info *info, *next;
	
	info = ts->list;
	while(info) {
		/* Save the "next" pointer now because info will be freed */
		next = info->next;
		
		handle = info->handle;
		info->ops->fini(info);
		if (handle)
			dlclose(handle);
		
		info = next;
	}
}

void tswarp_free(struct ts_wrapper * tswrap)
{
	int i;
	for( i = 0; i < tswrap->slot_count; i++)
		tswarp_free_tsdev(&tswrap->slot[i].ts);
	free(tswrap);
}

static int process_slot( struct ts_slot * slot, struct touch_point_container * tp, int i)
{
	struct ts_sample s;
	struct tslib_module_info * mod = slot->ts.list;
	int ret;

	slot->s.x = tp->processed.x;
	slot->s.y = tp->processed.y;
	slot->s.tv = tp->processed.time;
	if( !tp->processed.down ){
		slot->s.pressure = 0;
	}else{
		if( tp->processed.p != NOT_YET_WRITTEN )
			slot->s.pressure = tp->processed.p;
		else
			slot->s.pressure = 1;
	}
	pr_dbg("%s %d -- % 2d -- x: %d y:%d p:%d\n", __func__, __LINE__,i, slot->s.x, slot->s.y, slot->s.pressure);

	ret = mod->ops->read( mod, &s, 1);
	//tswrap_read( mod, &s, 1);

	pr_dbg("%s %d -- % 2d -- x: %d y:%d p:%d ret: %d\n", __func__, __LINE__,i, s.x, s.y, s.pressure, ret);
	if(ret ){
		tp->processed.x = s.x;
		tp->processed.y = s.y;
		tp->processed.time = slot->s.tv;
		if(  !s.pressure ){
			tp->processed.down = 0;
			if( tp->processed.p != NOT_YET_WRITTEN )
				tp->processed.p = 0;
		}else{
			tp->processed.down = 1;
			if( tp->processed.p != NOT_YET_WRITTEN )
				tp->processed.p  = s.pressure;
		}
	}else
	{
		// We 'drop' the sample with just copying the previous
		memcpy( &tp->processed, &tp->last_send, sizeof(struct touch_point));
	}
}

int tswrap_process( struct ts_wrapper * tswrap, struct touch_report * report)
{
	int i, ret;

	if(single_touch && report->single_touch->updated )
		ret = process_slot( &tswrap->slot[0], report->single_touch, -1);

	if(multi_touch)
		for( i = 0; i < report->max_tp_cnt; i++){
			if( report->mt_touch[i].updated ){
				ret = process_slot( &tswrap->slot[i+1], &(report->mt_touch[i]), i);
			}
	}

	return 0;
}

static const struct tslib_ops __ts_input_ops = {
	.read	= tswrap_read,
	.fini	= tswrap_fini,
};

TSAPI struct tslib_module_info *input_mod_init(struct tsdev *dev, const char *params)
{
	struct tslib_module_info * info = malloc( sizeof(struct tslib_module_info));
	if( !info)
	{
		pr_err("Failed to allocate memory\n");
		return 0;
	}
	info->dev = dev;
	info->ops = &tswrap_ops;
	return info;
}
TSLIB_MODULE_INIT(input_mod_init);
