/*
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
//#####DESCRIPTIONBEGIN####
//
// Author(s):    mow, av, tj
// Date:         2009-06-24
// Purpose:
// Description:
//
//####DESCRIPTIONEND####
//
//==========================================================================

#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <getopt.h>
#include <string.h>
#include <unistd.h>
#include "flash_config.h"
#include "file_config.h"
#include "ezxml.h"
#include "xconfig.h"

#undef _DO_TRACE

#define XML_TAG_CHECK_LENGTH	5
#define XML_INPUT_BUFFER_SIZE	512

/* 
HACK: The bionic implementation of getopt() does not work when the 
firts parameter is not in OPTS like the "list" in "xconfig list -p ...".
Throwing away the first argument fixes this issue.
*/
#ifdef __ANDROID__
#define getopt(ARGC,ARGV,OPTS) getopt((ARGC)-1, &((ARGV)[1]), (OPTS))
#endif

// #define DEBUG

#ifdef DEBUG
#define printf_d(args...) printf(args)
#else
#define	printf_d(args...) /* nothing */
#endif

#define	printf_e(args...) fprintf(stderr, args)

ezxml_t xml_cfgroot=NULL;		// root-node of in-RAM representation of fconfig-data
ezxml_t xml_fisroot=NULL;		// root-node of in-RAM representation of fis-data
ezxml_t xml_confroot=NULL;		// root-node of in-RAM XML-data
char *xml_confroot_str=NULL;
char *target_device=NULL;		// device for write-back
char *ts_conf=NULL;
char *pointercal=NULL;

options_t options={0, 0, NULL, NULL, NULL, NULL, 0, 0};

int usage()
{
	printf_e("Usage: $0 [command] [options] \n"
"\n"
"Script to manipulate the XML-configuration data of GuF i.MX Boards\n"
"inside the NAND Flash. \n"
"\n"
"xconfig expect one of the following commands to define its behavior: \n"
"  list           show the current contents of the XML-configuration as\n"
"                 obtained from NAND\n"
"  import         import an external XML-configuration file into the NAND\n"
"                 Flash\n"
"  addnode        add a new (empty) XML node\n"
"  delnode        delete XML node(s) and its/their children\n"
"  addattribute   add a new or change an existing attribute\n"
"  delattribute   delete an existing attribute\n"
"If no command and option given xconfig will display its version and exit.\n"
"\n"
"Syntax:\n"
#ifdef XCONFIG_TYPE_NAND
"  xconfig list [-v] [-f] [-p <path>] \n"
#else
"  xconfig list [-v] [-p <path>] \n"
#endif
"	-v           show whole (sub)tree instead of summary, the format is\n"
"	             suitable to be used as input to \"xconfig import\"\n"
"	-s			 search for a specific pattern\n"
"	-l			 show long path syntax instead of xml\n"
"	--node	     filter result by node name, basic wildcards are allowed\n"
"	--attribute	 filter result by attribute name, basic wildcards are allowed\n"
"                formate is either --attribute linux* matches all nodes that have\n"
"                          an attribute starting with linux\n"
"				 or --attribute key=\"value\" matching nodes that have an attribute 'key'\n"
"                          with the value 'value'\n"
#ifdef XCONFIG_TYPE_NAND
"	-f           by default xconfig operates on the <variables>-subtree\n"
"	             only, this option switches to the <flash>-subtree\n"
"				 defining the NAND partitions\n"
"				 *strongly discouraged* - Use the \"fis\" commands instead\n"
#endif
"	-p <path>    path to subtree, e.g.:\n"
"	             -p /configurationFile/variables/setting[@key='enable_serialdiag]\n"
"	             if omitted, the whole tree is shown\n"
"  xconfig import [-v] [-y] [-b] <file>\n"
"	-v           be verbose\n"
"	-y           assume yes - apply changes without user interaction\n"
"	<file>       path to file to import\n"
"	-b           deprecated - do not use\n"
#ifdef XCONFIG_TYPE_NAND
"  xconfig addnode [-f] [-y] -p <path> -n <name>\n"
"	-f           same as above\n"
#else
"  xconfig addnode [-y] -p <path> -n <name>\n"
#endif
"	-y           same as above\n"
"	-p <path>    path to parent node\n"
"	-n <name>    name of new node\n"
#ifdef XCONFIG_TYPE_NAND
"  xconfig delnode [-f] [-y] -p <path>\n"
"	-f           same as above\n"
#else
"  xconfig delnode [-y] -p <path>\n"
#endif
"	-y           same as above\n"
"	-p <path>    path to node\n"
#ifdef XCONFIG_TYPE_NAND
"  xconfig addattribute [-f] [-y] -p <path> -n <attr-name> -v <value>\n"
"	-f           same as above\n"
#else
"  xconfig addattribute [-y] -p <path> -n <attr-name> -v <value>\n"
#endif
"	-y           same as above\n"
"	-p <path>    path to parent node\n"
"	-n <attr-name>   name of the attribute\n"
"	-v <value>   new value of attribute\n"
#ifdef XCONFIG_TYPE_NAND
"  xconfig delattribute [-f] [-y] -p <path> -n <attr-name>\n"
"	-f           same as above\n"
#else
"  xconfig delattribute [-y] -p <path> -n <attr-name>\n"
#endif
"	-y           same as above\n"
"	-p <path>    path to parent node\n"
"	-n <attr-name>   name of the attribute\n"
"  xconfig listattributes -p <path>\n"
"	-p <path>    path to node\n"
"  xconfig getattribute -p <path> -n <attr-name>\n"
"	-p <path>    path to parent node\n"
"	-n <attr-name>   name of the attribute\n");
	return 0;
}

int parseargs(int argc, char *argv[])
{
	int opt;
	int i;
	if (argc == 1) {
		usage();
		return 0;
	}
	
	printf_d("Start parsing args: ");
	for (i = 0; i < argc; i++)
		printf_d("%s ", argv[i]);
	printf_d("\n");
	
	char *command = argv[1];
	if ( ! strcmp(command, "list")) {
		static struct option long_options[] = {
            {"node",  required_argument, 0,  0 },
            {"attribute",  required_argument, 0,  0 },
            {0,         0,                 0,  0 }
        };
		int longoption_idx;
		options.command = 1;
		while ((opt = getopt_long(argc, argv, "vflp:", long_options, &longoption_idx)) != -1) {
			switch (opt) {
			case 0:
				if(!optarg)
				{
					usage();
					return 1;
				}

				switch(longoption_idx)
				{
				case 0:
					options.node_pattern = optarg;
					break;
				case 1:
					options.attribute_pattern = strtok(optarg, "=");
					options.attribute_value = strtok(NULL, "=");
					printf_d(" Looking for attr '%s' with value '%s'\n", options.attribute_pattern, options.attribute_value );
					break;
				}
				break;
			case 'v':
				options.verbose = 1;
				printf_d("verbose\n");
				break;
			case 'l':
				options.longpath = 1;
				printf_d("verbose\n");
				break;
#ifdef XCONFIG_TYPE_NAND
			case 'f':
				options.flash = 1;
				printf_d("flash\n");
				break;
#endif
			case 'p':
				options.path = optarg;
				if ( ! strncmp(optarg,"/configurationFile", 18) ) {
					options.path+=18;
					printf_d("trunk'd path\n");
				}
				printf_d("path\n");
				break;
			default: /* '?' */
				printf_e("Unknown option...%c\n", opt);
				usage();
				return 1;
			}
		}
	} else if ( ! strcmp(command, "addnode")) {
		options.command = 2;
		while ((opt = getopt(argc, argv, "vyp:n:")) != -1) {
			switch (opt) {
			case 'v':
				options.verbose = 1;
				printf_d("verbose\n");
				break;
			case 'y':
				options.assumeyes = 1;
				printf_d("assume_yes\n");
				break;
			case 'p':
				options.path = optarg;
				if ( ! strncmp(optarg,"/configurationFile", 18) ) {
					options.path+=18;
					printf_d("trunk'd path\n");
				}
				printf_d("path=%s\n", options.path);
				break;
			case 'n':
				options.name = optarg;
				printf_d("name=%s\n", options.name);
				break;
			default: /* '?' */
				printf_e("Unknown option...\n");
				usage();
				return 1;
			}
		}
	} else if ( ! strcmp(command, "delnode")) {
		options.command = 3;
		while ((opt = getopt(argc, argv, "vyp:")) != -1) {
			switch (opt) {
			case 'v':
				options.verbose = 1;
				printf_d("verbose\n");
				break;
			case 'y':
				options.assumeyes = 1;
				printf_d("assume_yes\n");
				break;
			case 'p':
				options.path = optarg;
				if ( ! strncmp(optarg,"/configurationFile", 18) ) {
					options.path+=18;
					printf_d("trunk'd path\n");
				}
				printf_d("path=%s\n", options.path);
				break;
			default: /* '?' */
				printf_e("Unknown option...\n");
				usage();
				return 1;
			}
		}
	} else if ( ! strcmp(command, "addattribute")) {
		options.command = 4;
		while ((opt = getopt(argc, argv, "fyp:n:v:")) != -1) {
			switch (opt) {
#ifdef XCONFIG_TYPE_NAND
			case 'f':
				options.flash = 1;
				printf_d("flash\n");
				break;
#endif
			case 'y':
				options.assumeyes = 1;
				printf_d("assume_yes\n");
				break;
			case 'p':
				options.path = optarg;
				if ( ! strncmp(optarg,"/configurationFile", 18) ) {
					options.path+=18;
					printf_d("trunk'd path\n");
				}
				printf_d("path=%s\n", options.path);
				break;
			case 'n':
				options.name = optarg;
				printf_d("name=%s\n", options.name);
				break;
			case 'v':
				options.value = optarg;
				printf_d("value=%s\n", options.value);
				break;
			default: /* '?' */
				printf_e("Unknown option...\n");
				usage();
				return 1;
			}
		}
	} else if ( ! strcmp(command, "delattribute")) {
		options.command = 5;
		while ((opt = getopt(argc, argv, "fyp:n:")) != -1) {
			switch (opt) {
#ifdef XCONFIG_TYPE_NAND
			case 'f':
				options.flash = 1;
				printf_d("flash\n");
				break;
#endif
			case 'y':
				options.assumeyes = 1;
				printf_d("assume_yes\n");
				break;
			case 'p':
				options.path = optarg;
				if ( ! strncmp(optarg,"/configurationFile", 18) ) {
					options.path+=18;
					printf_d("trunk'd path\n");
				}
				printf_d("path=%s\n", options.path);
				break;
			case 'n':
				options.name = optarg;
				printf_d("name=%s\n", options.name);
				break;
			default: /* '?' */
				printf_e("Unknown option...\n");
				usage();
				return 1;
			}
		}
	} else if ( ! strcmp(command, "getattribute")) {
		options.command = 6;
		while ((opt = getopt(argc, argv, "p:n:")) != -1) {
			switch (opt) {
			case 'p':
				options.path = optarg;
				if ( ! strncmp(optarg,"/configurationFile", 18) ) {
					options.path+=18;
					printf_d("trunk'd path\n");
				}
				printf_d("path=%s\n", options.path);
				break;
			case 'n':
				options.name = optarg;
				printf_d("name=%s\n", options.name);
				break;
			default: /* '?' */
				printf_e("Unknown option...\n");
				usage();
				return 1;
			}
		}
	} else if ( ! strcmp(command, "listattributes")) {
		options.command = 7;
		while ((opt = getopt(argc, argv, "p:")) != -1) {
			switch (opt) {
			case 'p':
				options.path = optarg;
				if ( ! strncmp(optarg,"/configurationFile", 18) ) {
					options.path+=18;
					printf_d("trunk'd path\n");
				}
				printf_d("path=%s\n", options.path);
				break;
			default: /* '?' */
				printf_e("Unknown option...\n");
				usage();
				return 1;
			}
		}
	} else if ( ! strcmp(command, "import")) {
		options.command = 8;
		while ((opt = getopt(argc, argv, "vfyb:")) != -1) {
			switch (opt) {
			case 'v':
				options.verbose = 1;
				printf_d("verbose\n");
				break;
			case 'y':
				options.assumeyes = 1;
				printf_d("assume_yes\n");
				break;
			case 'b':
				options.base = optarg;
				printf_d("base=%s\n", options.base);
				break;
			default: /* '?' */
				printf_e("Unknown option...\n");
				usage();
				return 1;
			}
		}
		if ( ! options.base && argc > (optind+1))
			options.base = argv[optind+1];
	} else {
		printf_e("Unknown command: %s\n", command);
		return 1;
	}
	printf_d("Found command: %i\n", options.command);
	return 0;
}

/******************************************
 *** Internal helper functions
 ******************************************/
/*****************************************************************
 *Function name	: find_node
*/
/**
 * @brief	Checks if a given XML-node is already on the node-stack
 *
 * @param	xml			XML-node to search for
 * @param	head		head of node-stack
 *
 * @return	1 if element is already on node-stack,  otherwise
 */
static int find_node(ezxml_t xml, node_stack_t *head)
{
	for (; head; head = head->next)
		if (head->node == xml)
			return 1;

	return 0;
}

/*****************************************************************
 *Function name	: push_node
*/
/**
 * @brief	Push new element on node-stack
 *
 * @param	xml			XML-node to push on node-stack
 * @param	phead		head of node-stack
 *
 * @return	1 if element was pushed successfully,  otherwise
 */
static int push_node(ezxml_t xml, node_stack_t **phead)
{
	if (!phead)
		return 0;

	// only add node, if it is not already present
	if (!find_node(xml, *phead))
	{
		node_stack_t *new_node = (node_stack_t *)malloc(sizeof(node_stack_t));
		if (!new_node)
		{
			printf_e("ERROR: xconfig out-of-memory\n");
			return 0;
		}
		new_node->node = xml;
		new_node->next = *phead;
		*phead = new_node;
	}
	return 1;
}

/*****************************************************************
 *Function name	: pop_node
*/
/**
 * @brief	Pop top element from temporary node-stack
 *
 * @param	phead		head of node-stack
 *
 * @return	Top element from node-stack, if stack was not empty. NULL otherwise
 */
static node_stack_t *pop_node(node_stack_t **phead)
{
	node_stack_t *cur = NULL;

	if (!phead)
		return NULL;

	cur = *phead;
	if (cur)
		*phead = cur->next;

	return cur;
}


/*****************************************************************
 *Function name	: filter_xml_node
*/
/**
 * @brief	Filters an xml node by the node_pattern attribute_pattern and attribute_value
 *
 * @param	node		XML-node to be printed
 * @return	True/False
 */
static int filter_xml_node(ezxml_t node )
{
	char **attrs = NULL;
	if (!node)
		return 0;
	
	if( options.node_pattern)
	{
		if( fnmatch( options.node_pattern, ezxml_name(node), 0))
		{
			printf_d("Pattern '%s' does not match '%s'\n", options.node_pattern, ezxml_name(node));
			return 0;	
		}
	}
	if( options.attribute_pattern)
	{
		int match = 0;
		
		for (attrs = ezxml_attr_next(node, NULL);
			attrs && attrs[0] && attrs[1];
			attrs = ezxml_attr_next(node, attrs))
		{
			if( 0 == fnmatch( options.attribute_pattern, attrs[0] , 0))
			{
				if( !options.attribute_value 
				|| ( 0 == fnmatch( options.attribute_value, attrs[1] , 0)))
				{
					match = 1;
					break;
				}
			}
		}
		if(!match)
			return 0;
	}
	return 1;
}

/*****************************************************************
 *Function name	: show_xml_node_pre_long
*/
/**
 * @brief	Prints the opening tag of an XML-node and its attributes
 *
 * @param	node		XML-node to be printed
 * @param	verbose		if 1, show all attributes
 *						if , only show identifying attributes
 *
 * @return	Nothing
 */
static void show_xml_node_pre_long(ezxml_t node, int verbose)
{
#define MAX_DEPTH 20
	int i;
	char **attrs = NULL;
	int bprint_ellipsis = 0;
	int level = ezxml_depth(node);
	ezxml_t parent[MAX_DEPTH];

	if(!filter_xml_node(node))
		return;

	if( level >= MAX_DEPTH ) 
	{
		printf("Path for node %s can't be displayed\n", ezxml_name(node));
		return;
	}

	// go down to parent
	i = level;
	parent[i] = node;
	while( i--)
		parent[i] = ezxml_parent(parent[i+1]);
	for(i=1;i<=level;i++)
		printf("/%s", ezxml_name(parent[i]));

	// print attributes
	for (attrs = ezxml_attr_next(node, NULL);
		 attrs && attrs[0] && attrs[1];
		 attrs = ezxml_attr_next(node, attrs))
	{
		if (verbose)
		{
			// in verbose-mode we show all attributes,
			// each one on a new line.
			printf("\n    ");

			// printf() has problems with long strings...
			if (strlen(attrs[1]) < 128)
				printf("%s=\"%s\"", attrs[0], attrs[1]);
			else
			{
				char *ptr = attrs[1];
				printf("%s=\"", attrs[0]);
				for (i = strlen(attrs[1]); i; i--)
					printf("%c", *ptr++);
				printf("\"");
			}
		}
		else
		{
			// in non-verbose mode we only show identifying attributes
			if (!strcmp(attrs[0], "name") ||
				!strcmp(attrs[0], "key"))
			{
				printf(" %s=\"%s\"", attrs[0], attrs[1]);
			}
			else
			{
				// if tag has other attributes, show '...'
				bprint_ellipsis = 1;
			}
		}
	}
	printf("%s \n", bprint_ellipsis ? " ... " : "");
}

/*****************************************************************
 *Function name	: show_xml_node_post_long
*/
/**
 * @brief	Prints the closing tag of an XML-node
 *
 * @param	node		XML-node to be printed
 * @param	verbose		if 1, show what's happening
 *
 * @return	Nothing
 */
static void show_xml_node_post_long(ezxml_t node, int verbose)
{
	return;
}

/*****************************************************************
 *Function name	: show_xml_node_pre
*/
/**
 * @brief	Prints the opening tag of an XML-node and its attributes
 *
 * @param	node		XML-node to be printed
 * @param	verbose		if 1, show all attributes
 *						if , only show identifying attributes
 *
 * @return	Nothing
 */
static void show_xml_node_pre(ezxml_t node, int verbose)
{
	int i;
	char **attrs = NULL;
	int bprint_ellipsis = 0;
	int level = ezxml_depth(node);

	if(!filter_xml_node(node))
		return;

	// print yourself
	for (i = 0; i < level; i++)
		printf("    ");

	printf("<%s", ezxml_name(node));

	// print attributes
	for (attrs = ezxml_attr_next(node, NULL);
		 attrs && attrs[0] && attrs[1];
		 attrs = ezxml_attr_next(node, attrs))
	{
		if (verbose)
		{
			// in verbose-mode we show all attributes,
			// each one on a new line.
			printf("\n");
			for (i = 0; i < (level+1); i++)
				printf("    ");

			// printf() has problems with long strings...
			if (strlen(attrs[1]) < 128)
				printf("%s=\"%s\"", attrs[0], attrs[1]);
			else
			{
				char *ptr = attrs[1];
				printf("%s=\"", attrs[0]);
				for (i = strlen(attrs[1]); i; i--)
					printf("%c", *ptr++);
				printf("\"");
			}
		}
		else
		{
			// in non-verbose mode we only show identifying attributes
			if (!strcmp(attrs[0], "name") ||
				!strcmp(attrs[0], "key"))
			{
				printf(" %s=\"%s\"", attrs[0], attrs[1]);
			}
			else
			{
				// if tag has other attributes, show '...'
				bprint_ellipsis = 1;
			}
		}
	}
	printf("%s>\n", bprint_ellipsis ? " ... " : "");
}

/*****************************************************************
 *Function name	: show_xml_node_post
*/
/**
 * @brief	Prints the closing tag of an XML-node
 *
 * @param	node		XML-node to be printed
 * @param	verbose		if 1, show what's happening
 *
 * @return	Nothing
 */
static void show_xml_node_post(ezxml_t node, int verbose)
{
	int i;
	int level = ezxml_depth(node);

	if(!filter_xml_node(node))
		return;

	// print closing tag
	for (i = 0; i < level; i++)
		printf("    ");
	printf("</%s>\n", ezxml_name(node));
}

/*****************************************************************
 *Function name	: select_xmlroot
*/
/**
 * @brief	Selects XML configuration tree to be used
 *
 * @param	use_fisroot		if 1, returns FIS-tree, otherwise returns fconfig-tree
 * @param	warn_fis		if 1, print warning about dangers of using the FIS-tree
 *
 * @return	Handle to selected XML configuration tree
 */
static ezxml_t select_xmlroot(int use_fisroot, int warn_fis)
{
	if (use_fisroot)
	{
		if (warn_fis)
			printf_e("Warning: direct manipulation of fis-data may render the device unusable!!!\n");
		return xml_fisroot;
	}

	return xml_cfgroot;
}

/*****************************************************************
 *Function name	: save_to_flash
*/
/**
 * @brief	Saves XML configuration tree to flash
 *
 * @param	use_fisroot		if 1, save FIS-tree, otherwise saves fconfig-tree
 * @param	assume_yes		if 1, don't query for confirmation of writing to flash
 *
 * @return	Nothing
 */
int save_to_flash(int use_fisroot, int assume_yes)
{
	char *xml_str;
	int ret=0;

#ifdef XCONFIG_TYPE_NAND
	ezxml_t xml_flash = NULL;

	// update generation information
	xml_flash = ezxml_child(xml_confroot, "flash");
	if (!xml_flash)	{
		ret=1;
	} else {
		char temp_buf[11];
		int img_generation = strtoul(ezxml_attr(xml_flash, "generation"), NULL, 0);
		printf_d("img_generation=%u\n", img_generation);
		sprintf(temp_buf, "%u", img_generation+1);
		ezxml_set_attr_d(xml_flash, "generation", temp_buf);
	}
#endif

	xml_str=ezxml_toxml(xml_confroot);

#ifdef XCONFIG_TYPE_NAND
	ret=xml_write_to_mtd(xml_str, assume_yes, target_device);
#else
	ret=xml_write_to_file(xml_str, ts_conf, pointercal, assume_yes);
#endif

	free(xml_str);
	if (!ret)
		printf("Configuration successfully written.\n");
	return ret;
}

/*****************************************************************
 *Function name	: xml_copy_node
*/
/**
 * @brief	Copies a node from import-file into the configuration tree
 *
 * @param	xml_root	handle node under which the import-node should be added
 * @param	xml_dest	if not NULL, node in configuration tree that should
 *						be overwritten. If NULL, a new node is allocated under
 *						xml_root.
 * @param	xml_src		source node in import-file that shall be copied
 *						into the configuration tree.
 * @param	verbose		if 1, show what's happening
 * @param	key			name of attribute that can be used to identify nodes
 *
 * @return	Nothing
 */
static void xml_copy_node(ezxml_t xml_root, ezxml_t xml_dest,
						  ezxml_t xml_src, int verbose, const char *key)
{
	ezxml_t xml_child;
	char **attrs = NULL;

	if (!xml_root || !xml_src || !key)
		return;

	if (verbose)
		printf("%s \"%s\"-tag \"%s\"\n", xml_dest ? "Replacing" : "Adding",
					ezxml_name(xml_src), ezxml_attr(xml_src, key));

	// if now destination node is given, create a new one under xml_root
	if (!xml_dest)
	{
		xml_dest = ezxml_add_child_d(xml_root, ezxml_name(xml_src), 0);
		if (!xml_dest)
		{
			printf_e("Error: Couldn't add \"%s\"-tag \"%s\"\n",
						ezxml_name(xml_src), ezxml_attr(xml_src, key));
			return;
		}
	}

	// copy all attributes
	for (attrs = ezxml_attr_next(xml_src, NULL);
		 attrs && attrs[0];
		 attrs = ezxml_attr_next(xml_src, attrs))
	{
		if (!attrs[1])
			ezxml_set_attr(xml_dest, attrs[0], NULL);		// remove attribute
		else
			ezxml_set_attr_d(xml_dest, attrs[0], attrs[1]); // add attribute
	}

	// copy all children
	while((xml_child = ezxml_child(xml_src, NULL)))
	{
		// copy xml_child into xml_dest, replacing existing sub-nodes
		// with identical names.
		xml_copy_node(xml_dest, ezxml_child(xml_dest, ezxml_name(xml_child)), xml_child, verbose, "key");

		ezxml_remove(xml_child);
	}
}

/*****************************************************************
 *Function name	: xml_find_attribute
*/
/**
 * @brief	Helper function of xml_import. Searches of an XML
 *			node with a given attribute/value combination.
 *			(Can be used to find "setting" or "partition" nodes).
 *
 * @param	xml_root	handle node where to start searching
 * @param	name		name of attribute to search for
 * @param	value		value of attribute to search for
 *
 * @return	Handle to node containg name/value-attribute, if found.
 *			NULL otherwise.
 */
static ezxml_t xml_find_attribute(ezxml_t xml_root, const char *name, const char *value)
{
	if (!name || !value)
		return NULL;

	for (; xml_root; xml_root = xml_root->next)
	{
		const char *my_value = ezxml_attr(xml_root, name);
		if (my_value && !strcmp(my_value, value))
			break;
	}

	return xml_root;
}

/*****************************************************************
 *Function name	: xml_import_unsupported
*/
/**
 * @brief	Helper function of xml_import to handle unsupported import data
 *
 * @param	xml_root	handle to some node of configuration tree
 * @param	xml_import	handle to unsupported node of import file
 * @param	verbose		if 1, show what is happening
 *
 * @return	None
 */
static void xml_import_unsupported(ezxml_t xml_root, ezxml_t xml_import, int verbose)
{
	printf_e("Warning: Import of \"%s\"-tag not supported. Skipped.\n",
			    ezxml_name(xml_import));
}
/*****************************************************************
 *Function name	: xml_import_variables
*/
/**
 * @brief	Helper function of xml_import to handle importing data
 *			into a "variables"-node
 *
 * @param	xml_root	handle to "variables"-node of configuration tree
 * @param	xml_import	handle to "variables"-node of import file
 * @param	verbose		if 1, show what is happening
 *
 * @return	None
 */
static void xml_import_variables(ezxml_t xml_root, ezxml_t xml_import, int verbose)
{
	ezxml_t xml_child;

	if (!xml_import)	// nothing to do
		return;

	if (!xml_root)
	{
		xml_import_unsupported(xml_root, xml_import, verbose);
		return;
	}

	if (verbose)
		printf("Importing \"%s\"-tag\n", ezxml_name(xml_import));

	while((xml_child = ezxml_child(xml_import, NULL)))
	{
		if (!strcmp(ezxml_name(xml_child), "setting"))
		{
			// try to find a setting-tag with the same key
			ezxml_t xml_setting = xml_find_attribute(ezxml_child(xml_root, "setting"), "key", ezxml_attr(xml_child, "key"));

			// copy new setting into tree (either overwriting existing setting or allocating a new one)
			xml_copy_node(xml_root, xml_setting, xml_child, verbose, "key");
		}
		else if (!strcmp(ezxml_name(xml_child), "display"))
		{
			// try to find a display-tag and delete it
			ezxml_remove(ezxml_child(xml_root, "display"));

			// copy new display into tree (allocating a new display-node)
			xml_copy_node(xml_root, NULL, xml_child, verbose, "name");
		}
		else if (!strcmp(ezxml_name(xml_child), "display2"))
		{
			// try to find a display-tag and delete it
			ezxml_remove(ezxml_child(xml_root, "display2"));

			// copy new display into tree (allocating a new display-node)
			xml_copy_node(xml_root, NULL, xml_child, verbose, "name");
		}
		else if (!strcmp(ezxml_name(xml_child), "hdmi"))
		{
			// try to find a display-tag and delete it
			ezxml_remove(ezxml_child(xml_root, "hdmi"));

			// copy new display into tree (allocating a new display-node)
			xml_copy_node(xml_root, NULL, xml_child, verbose, "name");
		}
		else if (!strcmp(ezxml_name(xml_child), "logo-license"))
		{
			// try to find a logo-license-tag and delete it
			ezxml_remove(ezxml_child(xml_root, "logo-license"));

			// copy new logo-license into tree (allocating a new node)
			xml_copy_node(xml_root, NULL, xml_child, verbose, "license");
		}
		else if (!strcmp(ezxml_name(xml_child), "touch"))
		{
			// try to find a touch-tag and delete it
			ezxml_remove(ezxml_child(xml_root, "touch"));

			// copy new touch into tree (allocating a new node)
			xml_copy_node(xml_root, NULL, xml_child, verbose, "touch");
		}
		else if (!strcmp(ezxml_name(xml_child), "patch-list"))
		{
			// try to find a setting-tag with the same name
			ezxml_t xml_patch_list = xml_find_attribute(ezxml_child(xml_root, "patch-list"), "name", ezxml_attr(xml_child, "name"));

			// copy new patch-list into tree (allocating a new node)
			xml_copy_node(xml_root, xml_patch_list, xml_child, verbose, "name");
		}
		else if (!strcmp(ezxml_name(xml_child), "pll"))
		{
			// try to find a setting-tag with the same name
			ezxml_t xml_pll = xml_find_attribute(ezxml_child(xml_root, "pll"), "pll", ezxml_attr(xml_child, "pll"));

			// copy new patch-list into tree (allocating a new node)
			xml_copy_node(xml_root, xml_pll, xml_child, verbose, "pll");
		}
		else
		{
			xml_import_unsupported(xml_root, xml_child, verbose);
		}

		ezxml_remove(xml_child);
	}
}

/*****************************************************************
 *Function name	: xml_import_configurationFile
*/
/**
 * @brief	Helper function of xml_import to handle importing data
 *			into a "configurationFile"-node
 *
 * @param	xml_root	(top-level) handle to configuration tree
 * @param	xml_import	(top-level) handle to import file
 * @param	verbose		if 1, show what is happening
 *
 * @return	None
 */
static void xml_import_configurationFile(ezxml_t xml_import, int verbose)
{
	ezxml_t xml_child;

	if (!xml_import)	// we're done
		return;

	if (verbose)
		printf("Importing \"%s\"-tag\n", ezxml_name(xml_import));

	while((xml_child = ezxml_child(xml_import, NULL)))
	{
		if (!strcmp(ezxml_name(xml_child), "variables"))
			xml_import_variables(xml_cfgroot, xml_child, verbose);
		else
			xml_import_unsupported(xml_cfgroot, xml_child, verbose);

		ezxml_remove(xml_child);
	}
}

/*****************************************************************
 *Function name	: xml_list
*/
/**
 * @brief	List contents of XML configuration tree
 *
 * @param	argc	number of command-line arguments
 * @param	argv	list of command-line arguments
 *
 * @return	0 if printed, 1 if node path does not exist
 */
static int xml_list(int verbose, char *path, int use_fisroot)
{
    const char **pi = NULL;
    ezxml_t xml_root = xml_cfgroot;
    printf_d("\n==> xml_list(verbose= %i, path=%s, use_fisroot=%i)\n",
    		verbose, path, use_fisroot);
	xml_root = select_xmlroot(use_fisroot, 0);
    node_stack_t *matching_nodes = NULL;
    node_stack_t *cur_node = NULL;
	int count = 0;

	// in verbose mode we print processing instructions
	if (verbose)
		for (pi = ezxml_pi(xml_root, "xml"); *pi; pi++)
			printf("<?xml %s?>\n", *pi);

	// check if path exists by counting matching nodes
	ezxml_select_xpath(xml_root, path, 1,
				       (ezxml_node_func_t)push_node, (void *)&matching_nodes,
				       (ezxml_node_func_t)NULL, (void *)NULL);
	while ((cur_node = pop_node(&matching_nodes)))
	{
		count++;
		free(cur_node);
	}
	if (count == 0)
		return 1;

	// print XML-tree
	if( !options.longpath)
	ezxml_select_xpath(xml_root, path, 0,
				       (ezxml_node_func_t)show_xml_node_pre, (void *)verbose,
				       (ezxml_node_func_t)show_xml_node_post, (void *)verbose);
	else
	ezxml_select_xpath(xml_root, path, 0,
				       (ezxml_node_func_t)show_xml_node_pre_long, (void *)verbose,
				       (ezxml_node_func_t)show_xml_node_post_long, (void *)verbose);

	return 0;
}

/*****************************************************************
 *Function name	: xml_import
*/
/**
 * @brief	Import an XML configuration file from RAM into configuration tree
 *
 * @param	argc	number of command-line arguments
 * @param	argv	list of command-line arguments
 *
 * @return	None
 */
static void xml_import(int verbose, char *filename, int assume_yes)
{
    ezxml_t xml_import = NULL;
#ifdef XCONFIG_TYPE_FILE
	ezxml_t linux_driver_info = NULL;
#endif
    char *xml_import_str = NULL;
	FILE *input = NULL;
	int len, count, buffer_len = 0;

	if ( ! filename)
		input = stdin;
	else
		input = fopen(filename, "r");

	if (input == NULL) {
		printf_e("Could not open file\n");
		return;
	}

	// check for <?xml tag
	xml_import_str = malloc(XML_INPUT_BUFFER_SIZE);
	if ( ! xml_import_str ) {
		printf_e("Insufficient memory\n");
		if (input != stdin)
			fclose(input);
		return;
	}
	buffer_len = XML_INPUT_BUFFER_SIZE;
	count = fread(xml_import_str, 1, XML_TAG_CHECK_LENGTH, input);
	xml_import_str[XML_TAG_CHECK_LENGTH] = '\0';
	if ( count != XML_TAG_CHECK_LENGTH || ! strstr(xml_import_str, "<?xml") ) {
		printf_e("%s: No valid XML structure\n", filename?filename:"stdin");
		free(xml_import_str);
		if (input != stdin)
			fclose(input);
		return;
	}
	len = XML_TAG_CHECK_LENGTH;

	// read rest of the file
	do {
		count = fread(xml_import_str + len, 1, buffer_len - len, input);
		len += count;
		if (len == buffer_len) {
			buffer_len += XML_INPUT_BUFFER_SIZE;
			char *temp_str = realloc(xml_import_str, buffer_len);
			if ( ! temp_str ) {
				printf_e("Insufficient memory\n");
				free(xml_import_str);
				if (input != stdin)
					fclose(input);
				return;
			}
			xml_import_str = temp_str;
		}
	} while (count > 0);

	xml_import_str[len] = 0;

	if (input != stdin)
		fclose(input);

	if (verbose)
		printf("Importing %u bytes of XML-data from 0x%08x\n", strlen(xml_import_str), (unsigned int)xml_import_str);

	xml_import = ezxml_parse_str(xml_import_str, strlen(xml_import_str));
	if (strlen(ezxml_error(xml_import)) != 0) {
		printf_e("XML parsing error: %s\n", ezxml_error(xml_import));
		free(xml_import_str);
		return;
	}

#ifdef XCONFIG_TYPE_FILE
	/*
	 * If the file to be imported is a touch configuration containing a ts_conf
	 * tag, then we have to store the contents separately in /etc/shared/ts.conf.
	 */
	linux_driver_info = ezxml_child(ezxml_child(ezxml_child(
						ezxml_child(xml_import, "variables"), "touch"),
						"driver_info"), "linux");
	ts_conf = (char*) ezxml_attr(linux_driver_info, "ts_conf");
	if (ts_conf)
		ts_conf = strdup(ts_conf);
	// delete ts_conf attribute
	ezxml_set_attr(linux_driver_info, "ts_conf", NULL);
	/*
	 * Likewise, the pointercal tag is stored in /etc/shared/pointercal.
	 */
	pointercal = (char*) ezxml_attr(linux_driver_info, "pointercal");
	if (pointercal)
		pointercal = strdup(pointercal);
	// delete pointercal attribute
	ezxml_set_attr(linux_driver_info, "pointercal", NULL);
#endif

	if (xml_import && !strcmp(ezxml_name(xml_import), "configurationFile"))
		xml_import_configurationFile(xml_import, verbose);
	else
		printf_e("Error: Unsupported configuration file format.\n");

	ezxml_free(xml_import);
	if (xml_import_str)
		free(xml_import_str);
	save_to_flash(0, assume_yes);
}

/*****************************************************************
 *Function name	: xml_delnode
*/
/**
 * @brief	Delete an XML-node
 *
 * @param	argc	number of command-line arguments
 * @param	argv	list of command-line arguments
 *
 * @return	None
 */
static void xml_delnode(int verbose, char *path, int use_fisroot, int assume_yes)
{
    node_stack_t *matching_nodes = NULL;
    node_stack_t *cur_node = NULL;

	// find all selected nodes
	ezxml_select_xpath(select_xmlroot(use_fisroot, 1), path, 1,
				       (ezxml_node_func_t)push_node, (void *)&matching_nodes,
				       (ezxml_node_func_t)NULL, (void *)NULL);

	// delete them
	while ((cur_node = pop_node(&matching_nodes)))
	{
		printf("delete node \"%s\":0x%08x\n", ezxml_name(cur_node->node), (unsigned int)cur_node->node);
		ezxml_remove(cur_node->node);
		free(cur_node);
	}

	save_to_flash(use_fisroot, assume_yes);
}

/*****************************************************************
 *Function name	: xml_addnode
*/
/**
 * @brief	Add new sub-node to an XML-node
 *
 * @param	argc	number of command-line arguments
 * @param	argv	list of command-line arguments
 *
 * @return	None
 */
static void xml_addnode(int verbose, char *path, char *name, int use_fisroot, int assume_yes)
{
    node_stack_t *matching_nodes = NULL;
    node_stack_t *cur_node = NULL;

	// find all selected nodes
	ezxml_select_xpath(select_xmlroot(use_fisroot, 1), path, 1,
				       (ezxml_node_func_t)push_node, (void *)&matching_nodes,
				       (ezxml_node_func_t)NULL, (void *)NULL);

	// add a new node to each one
	while ((cur_node = pop_node(&matching_nodes)))
	{
		printf("add node \"%s\" to node \"%s\":0x%08x\n", name, ezxml_name(cur_node->node), (unsigned int)cur_node->node);
		ezxml_add_child_d(cur_node->node, name, 0);
		free(cur_node);
	}

	save_to_flash(use_fisroot, assume_yes);
}

/*****************************************************************
 *Function name	: xml_delattribute
*/
/**
 * @brief	Delete attribute of an XML-node
 *
 * @param	argc	number of command-line arguments
 * @param	argv	list of command-line arguments
 *
 * @return	None
 */
static void xml_delattribute(char *path, char *name, int use_fisroot, int assume_yes)
{
    node_stack_t *matching_nodes = NULL;
    node_stack_t *cur_node = NULL;

	// find all selected nodes
	ezxml_select_xpath(select_xmlroot(use_fisroot, 1), path, 1,
				       (ezxml_node_func_t)push_node, (void *)&matching_nodes,
				       (ezxml_node_func_t)NULL, (void *)NULL);

	// delete attribute in each one
	while ((cur_node = pop_node(&matching_nodes)))
	{
		printf("delete attribute \"%s\" from node \"%s\":0x%08x\n", name, ezxml_name(cur_node->node), (unsigned int)cur_node->node);
		ezxml_set_attr(cur_node->node, name, NULL);
		free(cur_node);
	}

	save_to_flash(use_fisroot, assume_yes);
}

/*****************************************************************
 *Function name	: xml_addattribute
*/
/**
 * @brief	Add new or modify existing attribute of an XML-node
 *
 * @param	argc	number of command-line arguments
 * @param	argv	list of command-line arguments
 *
 * @return	None
 */
static void xml_addattribute(char *path, char *name, char *value, int use_fisroot, int assume_yes)
{
    node_stack_t *matching_nodes = NULL;
    node_stack_t *cur_node = NULL;

	// find all selected nodes
	ezxml_select_xpath(select_xmlroot(use_fisroot, 1), path, 1,
				       (ezxml_node_func_t)push_node, (void *)&matching_nodes,
				       (ezxml_node_func_t)NULL, (void *)NULL);

	// add attribute to each one
	while ((cur_node = pop_node(&matching_nodes)))
	{
		printf("add attribute \"%s\"=\"%s\" to node \"%s\":0x%08x\n", name, value, ezxml_name(cur_node->node), (unsigned int)cur_node->node);
		ezxml_set_attr_d(cur_node->node, name, value);
		free(cur_node);
	}

	save_to_flash(use_fisroot, assume_yes);
}

/*****************************************************************
 *Function name	: xml_getattribute
*/
/**
 * @brief	Get value of an attribute
 *
 * @param	argc	number of command-line arguments
 * @param	argv	list of command-line arguments
 *
 * @return	0 if attribute exists, else 1
 */
static int xml_getattribute(char *path, char *name, int use_fisroot, int assume_yes)
{
    node_stack_t *matching_nodes = NULL;
    node_stack_t *cur_node = NULL;
	int count = 0;

	// find all selected nodes
	ezxml_select_xpath(select_xmlroot(use_fisroot, 1), path, 1,
				       (ezxml_node_func_t)push_node, (void *)&matching_nodes,
				       (ezxml_node_func_t)NULL, (void *)NULL);

	// display attribute value of each one
	while ((cur_node = pop_node(&matching_nodes)))
	{
		const char *value = ezxml_attr(cur_node->node, name);
		if (value != NULL) {
			count++;
			printf("%s\n", value);
		}
		free(cur_node);
	}
	return (count == 0);
}

/*****************************************************************
 *Function name	: xml_listattributes
*/
/**
 * @brief	List attributes of an XML-node
 *
 * @param	argc	number of command-line arguments
 * @param	argv	list of command-line arguments
 *
 * @return	0 if node exists, else 1
 */
static int xml_listattributes(char *path, char *name, int use_fisroot, int assume_yes)
{
    node_stack_t *matching_nodes = NULL;
    node_stack_t *cur_node = NULL;
	int count = 0;

	// find all selected nodes
	ezxml_select_xpath(select_xmlroot(use_fisroot, 1), path, 1,
				       (ezxml_node_func_t)push_node, (void *)&matching_nodes,
				       (ezxml_node_func_t)NULL, (void *)NULL);

	// display attributes of each one
	while ((cur_node = pop_node(&matching_nodes)))
	{
		char **attrs = NULL;
		count++;
		for (attrs = ezxml_attr_next(cur_node->node, NULL);
			 attrs && attrs[0] && attrs[1];
			 attrs = ezxml_attr_next(cur_node->node, attrs))
		{
			printf("%s\n", *attrs);
		}
		free(cur_node);
	}
	return (count == 0);
}

/**************************************************************
 * main
 */
int main(int argc, char *argv[])
{
	int ret=0;
	ret = parseargs(argc, argv);
	int len=0;
	if(ret)
		goto _exit;

#ifdef XCONFIG_TYPE_NAND
	// read xml-data from NAND
	ret = xml_read_from_mtd(&xml_confroot_str, &target_device);
#else
	// read xml-data from file
	ret = xml_read_from_file(&xml_confroot_str);
#endif
	if (ret ||  (! (xml_confroot_str)) ) {
		printf_e("Error obtaining XML-data\n");
		goto _exit;
	}
	printf_d("Got XML Data\n");
	len = strlen(xml_confroot_str);
	xml_confroot = ezxml_parse_str(xml_confroot_str, len);
	printf_d("XML-Error: %s\n", ezxml_error(xml_confroot));

	// split into xml_cfgroot (<variables>...</variables>) and
	//            xml_fisroot (<flash>...</flash>
	xml_cfgroot = ezxml_child(xml_confroot, "variables");
	if (!xml_cfgroot) {
		printf_e("No <variables> tag found, aborting\n");
		ret=1;
		goto _exit;
	}
	printf_d("Got <variables> subtree \n");
#ifdef XCONFIG_TYPE_NAND
	xml_fisroot = ezxml_child(xml_confroot, "flash");
	if (!xml_fisroot) {
		printf_e("No <flash> tag found, aborting\n");
		ret=1;
		goto _exit;
	}
	printf_d("Got <flash> subtree \n");
#endif

	// eval options
	switch (options.command) {
	case 1: 	// list
		// set the default path if none given
		if (! options.path )
			options.path = ( options.flash ) ? "/flash" : "/variables" ;
		ret = xml_list(options.verbose, options.path, options.flash);
		break;
	case 2: 	// addnode
		if (! options.path ) {
			printf_e("No path given! Use -p to specify the path to the node. \n");
			ret=1;
			goto _exit;
		}
		if (! options.name ) {
			printf_e("No name given! Use -n to specify the name of the node. \n");
			ret=1;
			goto _exit;
		}
		xml_addnode(options.verbose, options.path, options.name, options.flash, options.assumeyes);
		break;
	case 3: 	// delnode
		if (! options.path ) {
			printf_e("No path given! Use -p to specify the path to the node. \n");
			ret=1;
			goto _exit;
		}
		xml_delnode(options.verbose, options.path, options.flash, options.assumeyes);
		break;
	case 4: 	// addattribute
		if (! options.path ) {
			printf_e("No path given! Use -p to specify the path to the attribute. \n");
			ret=1;
			goto _exit;
		}
		if (! options.name ) {
			printf_e("No name given! Use -n to specify the name of the attribute. \n");
			ret=1;
			goto _exit;
		}
		if (! options.value ) {
			printf_e("No value given! Use -v to specify the value of the attribute. \n");
			ret=1;
			goto _exit;
		}
		xml_addattribute(options.path, options.name, options.value, options.flash, options.assumeyes);
		break;
	case 5: 	// delattribute
		if (! options.path ) {
			printf_e("No path given! Use -p to specify the path to the attribute. \n");
			ret=1;
			goto _exit;
		}
		if (! options.name ) {
			printf_e("No name given! Use -n to specify the name of the attribute. \n");
			ret=1;
			goto _exit;
		}
		xml_delattribute(options.path, options.name, options.flash, options.assumeyes);
		break;
	case 6: 	// getattribute
		if (! options.path ) {
			printf_e("No path given! Use -p to specify the path to the attribute. \n");
			ret=1;
			goto _exit;
		}
		if (! options.name ) {
			printf_e("No name given! Use -n to specify the name of the attribute. \n");
			ret=1;
			goto _exit;
		}
		ret = xml_getattribute(options.path, options.name, options.flash, options.assumeyes);
		break;
	case 7: 	// listattributes
		if (! options.path ) {
			printf_e("No path given! Use -p to specify the path to the attribute. \n");
			ret=1;
			goto _exit;
		}
		ret = xml_listattributes(options.path, options.name, options.flash, options.assumeyes);
		break;
	case 8: 	// import
		if (isatty(fileno(stdin)) && ! options.base ) {
			printf_e("No filename given! Append the file name to import or use redirection to stdin. \n");
			ret=1;
			goto _exit;
		}
		if ( ! options.base )
			options.assumeyes = 1;
		xml_import(options.verbose, options.base, options.assumeyes);
		break;
	default: 	// fail
		ret=1;
	}

	// clean up
_exit:
	if (pointercal)
		free(pointercal);
	if (ts_conf)
		free(ts_conf);
	if (xml_confroot)
		ezxml_free(xml_confroot);
	if (target_device)
		free(target_device);
	if (xml_confroot_str)
		free(xml_confroot_str);
	exit(ret);
}
