# Common include file for linux-fsl kernel recipes

SECTION = "kernel"
LICENSE = "GPLv2"

inherit kernel
inherit release

MODULE_TARBALL_DEPLOY = "0"

do_deploy_append(){
	install -m 0644 ${S}/vmlinux ${DEPLOYDIR}/vmlinux
}

do_release() {
	install -m 0644 ${DEPLOYDIR}/${KERNEL_IMAGE_SYMLINK_NAME}.bin ${RELEASEDIR}/${RELEASE_PREFIX}linuximage.bin

	install -d ${RELEASEDIR}/debug
	install -m 0644 ${DEPLOYDIR}/vmlinux ${RELEASEDIR}/debug/${RELEASE_PREFIX}vmlinux

	if test -n "${KERNEL_DEVICETREE}"; then
		for DTB in ${KERNEL_DEVICETREE}; do
			DTB_BASE_NAME=`basename ${DTB} .dtb`
			DTB_SYMLINK_NAME=`echo ${KERNEL_IMAGE_SYMLINK_NAME} | sed "s/${MACHINE}/${DTB_BASE_NAME}/g"`
			install -m 0644 ${DEPLOYDIR}/${DTB_SYMLINK_NAME}.dtb ${RELEASEDIR}/${RELEASE_PREFIX}${DTB_BASE_NAME}.dtb
		done
	fi
}
addtask do_release after do_deploy

# This function is not part of the normal sequence as the same is done in
# install append. It has been copied to a separate function here only so that
# the device tree can be compiled independently of the rest.
do_compile_dt() {
	if test -n "${KERNEL_DEVICETREE}"; then
		for DTB in ${KERNEL_DEVICETREE}; do
			if echo ${DTB} | grep -q '/dts/'; then
				bbwarn "${DTB} contains the full path to the the dts file, but only the dtb name should be used."
				DTB=`basename ${DTB} | sed 's,\.dts$,.dtb,g'`
			fi
			DTB_BASE_NAME=`basename ${DTB} .dtb`
			DTB_NAME=`echo ${KERNEL_IMAGE_BASE_NAME} | sed "s/${MACHINE}/${DTB_BASE_NAME}/g"`
			DTB_SYMLINK_NAME=`echo ${KERNEL_IMAGE_SYMLINK_NAME} | sed "s/${MACHINE}/${DTB_BASE_NAME}/g"`
			DTB_PATH="${B}/arch/${ARCH}/boot/dts/${DTB}"
			oe_runmake ${DTB}
			# Also decompile the script again to see the results of the processing
			dtc -I dtb -O dts -o ${DTB_PATH}.dts ${DTB_PATH}
		done
	fi
}
addtask compile_dt

# Install gfeeprom.h in order to add it to SDK
do_install_append() {
	install -d ${D}${includedir}/linux/i2c
	install -m 644 ${S}/include/linux/i2c/gfeeprom.h ${D}${includedir}/linux/i2c/gfeeprom.h
}
FILES_kernel-dev_append = " ${includedir} "
