DESCRIPTION = "Shiboken is a plugin (front-end) for Generator Runner and a runtime library. It generates \
bindings for C++ libraries using CPython source code."

HOMEPAGE = "http://www.pyside.org"

DEPENDS = "apiextractor-native generatorrunner-native shiboken-native python3 qt4-embedded"
LICENSE = "GPLv2"
LIC_FILES_CHKSUM = "file://COPYING;md5=34337af480a8c452bfafe22a78fa20cb"

SRC_URI = "git://gitorious.org/pyside/shiboken.git;protocol=git;tag=eb293c2839cfbc726f0e085e1435c94b6f6561f9 \
			  file://MacroPushRequiredVars.cmake \
			  file://rename-shiboken-pkg.patch \
			  file://FindQt4.cmake \
			  "

S = "${WORKDIR}/git"
inherit cmake pkgconfig python-dir

SRC_URI[md5sum] = "946e8988e5f4c4bd62e774407fa80fee"
SRC_URI[sha256sum] = "82c6c24dc55458ed047eba9fe700894a3347cd53462b21a97b7b5f9180b2a896"

EXTRA_OECMAKE += "  \
				-DUSE_PYTHON3:BOOL=on          \
				-DPYTHON3_EXECUTABLE=${STAGING_BINDIR_NATIVE}/python3-native/python3       \
				-DPYTHON3_DBG_EXECUTABLE=${STAGING_BINDIR_NATIVE}/python3-native/python3          \
				-DPYTHON3_INCLUDE_DIR=${STAGING_INCDIR}/python3.3m       \
				-DPYTHON3_LIBRARY=${STAGING_LIBDIR}/libpython3.so          \
				-DBUILD_TESTS:BOOL=OFF         \
				-DCMAKE_LIBRARY_PATH=${STAGING_LIBDIR}/  \
				\
				 -Wno-dev \
				\
				-DQT_LIBRARY_DIR=${STAGING_LIBDIR}         \
				-DQT_LIBINFIX="E" \
				-DQT_DIR_NAME="qtopia" \
				-DQT_BINARY_DIR=${STAGING_BINDIR_NATIVE}/        \
				-DQT_QMAKE_EXECUTABLE=${STAGING_BINDIR_NATIVE}/qmake2        \
				-DQT_QTCORE_FOUND=yes \
				-DQT_QTCORE_LIBRARY=${STAGING_LIBDIR}/libQtCoreE.so         \
				-DQT_QTGUI_FOUND=yes \
				-DQT_QTGUI_LIBRARY=${STAGING_LIBDIR}/libQtGuiE.so         \
				-DQT_QTXML_FOUND=yes \
				-DQT_QTXML_LIBRARY=${STAGING_LIBDIR}/libQtXmlE.so         \
				-DQT_USE_QTXMLPATTERNS:BOOL=on \
				-DQT_QTXMLPATTERNS_FOUND=yes \
				-DQT_QTXMLPATTERNS_LIBRARY=${STAGING_LIBDIR}/libQtXmlPatternsE.so         \
				-DQT_QTNETWORK_FOUND=yes \
				-DQT_QTNETWORK_LIBRARY=${STAGING_LIBDIR}/libQtNetworkE.so         \
			"

#				-DQT_INCLUDE_DIR=${STAGING_INCDIR}/qtopia \
#				-DQT_QTCORE_INCLUDE_DIR=${STAGING_INCDIR}/qtopia/QtCore \
#


# The following exports are needed to let the cmake build configuration succeed without
# errors when detecting the correct python version
export HOST_SYS
export BUILD_SYS
export STAGING_LIBDIR
export STAGING_INCDIR

addtask do_fix_generator_names after do_patch before do_configure

do_fix_generator_names() {
    mv ${S}/generator/shiboken ${S}/generator/shiboken-src
    ln -s ${STAGING_BINDIR_NATIVE}/shiboken ${S}/generator/shiboken
}

do_configure_prepend() {
   cp ${WORKDIR}/MacroPushRequiredVars.cmake ${S}/cmake/Modules/MacroPushRequiredVars.cmake
   cp ${WORKDIR}/FindQt4.cmake ${S}/cmake/Modules/FindQt4.cmake
}

do_compile_prepend() {
    ln -sf ${STAGING_BINDIR_NATIVE}/shiboken ${S}/../build/generator/shiboken
}

FILES_${PN}-dbg += "${libdir}/python3.3/site-packages/.debug/shiboken*"
FILES_${PN}-dev += "${libdir}/cmake/ ${libdir}/pkgconfig"
FILES_${PN} += "${libdir}/python3.3/site-packages/shiboken*"


# NOTE: This needs to be appended to do_configure as pkgconfig.bbclass uses
# do_install_prepend for it's fixups and we need to run before it!

do_configure_append() {
   # Fixup generated *.cmake and *.pc files for wrong paths
   for i in `find ${S}/../build/data/ -name "ShibokenConfig.cmake" -type f` ; do \
	    sed -i \
			-e 's:/usr/lib:${STAGING_LIBDIR}:g' \
			-e 's:/usr/include:${STAGING_INCDIR}:g' \
			-e 's:/usr/bin:${STAGING_BINDIR}:g' \
			$i; \
   done
}

#   # We need do this here a second time (pkgconfig.bbclass already replaces the -L.. and
#   # -I .. ones) as there are additional variables for python in the pkgconfig file
#   for i in `find ${S}/data -name "*.pc" -type f` ; do \
#      sed -i -e 's:${STAGING_BINDIR_NATIVE}:${bindir}:g' \
#         -e 's:${STAGING_INCDIR}:${includedir}:g' \
#         -e 's:${STAGING_LIBDIR}:${libdir}:g' \
#         -e 's:${STAGING_INCDIR_NATIVE}:${includedir}:g' \
#         -e 's:${STAGING_LIBDIR_NATIVE}:${libdir}:g' \
#         -e 's:-lshiboken:-lshiboken-${PYTHON_DIR}:g' \
#         $i
#   done

#do_configure_append (){
#	ln -sf 
#}
#inherit cmake 

